import React, { Component } from 'react'
import 'primeflex/primeflex.css';
// import { Link } from 'react-router-dom/cjs/react-router-dom.min';
import _ from 'lodash';
import ViewTable from '../../components/ViewTable';
import CycleService from '../../services/cycle.service';
import UnitConversion from '../../utils/unit.converter';
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import UIConstants from '../../utils/ui.constants';
import UtilService from '../../services/util.service';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
/* eslint-disable no-unused-expressions */

class CycleList extends Component {
    lsTableName = "cycle_list";
    lsKeySortColumn='cycleSortData';
    descendingColumn=['start date']; // Values should be lower case
    constructor(props) {
        super(props)
        this.state = {
            cyclelist: [],
            paths: [{
                "View": "/cycle/view",
            }],
            isLoading: true,
            userrole:  {
                userRolePermission : {}
            }
        }
        this.projectCategory = ['regular', 'user_shared_support'];
        this.periodCategory = ['long_term'];
        this.defaultcolumns = [{
            id: "Cycle Code",
            start: {
                name: "Start Date",
                filter: "date",
                format: UIConstants.CALENDAR_DEFAULTDATE_FORMAT
            },
            stop: {
                name: "End Date",
                filter: "date",
                format: UIConstants.CALENDAR_DEFAULTDATE_FORMAT
            },
            totalProjects: {
                name: 'No.of Projects',
                filter: "range"
            },
            observingTime: {
                name: "Lofar Observing Time (Hrs)",
                filter: "range"
            },
            processingTime: {
                name: "Lofar Processing Time (Hrs)",
                filter: "range"
            },
            ltaResources: {
                name: "Lofar LTA Resources(TB)",
                filter: "range"
            }
        }];
        this.optionalcolumns = [{
            duration: {
                name: "Duration (Days)",
                filter: "range",
                format: UIConstants.CALENDAR_TIME_FORMAT
            },
            support: {
                name: "Lofar Support (Hrs)",
                filter: "range"
            },
            longterm: {
                name: "Long Term Projects",
                filter: "range"
            },
            regularProjects: {
                name: "No.of Regular Projects",
                filter: "range"
            },
            observingTimeDDT: {
                name: "Lofar Observing Time Commissioning (Hrs)",
                filter: "range"
            },
            observingTimePrioA: {
                name: "Lofar Observing Time Prio A (Hrs)",
                filter: "range"
            },
            observingTimePrioB: {
                name: "Lofar Observing Time Prio B (Hrs)",
                filter: "range"
            },
            actionpath: "actionpath"
        }];
        this.columnOrder = [
            "Action", "Cycle Code", "Start Date", "End Date", "Duration (Days)", "No.of Projects",
            "No.of Regular Projects", "Lofar Observing Time (Hrs)", "Lofar Processing Time (Hrs)",
            "Lofar LTA Resources(TB)", "Lofar Support (Hrs)", "Lofar Observing Time Commissioning (Hrs)",
            "Lofar Observing Time Prio A (Hrs)", "Lofar Observing Time Prio B (Hrs)"
        ]
        this.columnclassname = [{
            "Cycle Code": "filter-input-75",
            "Duration (Days)": "filter-input-50",
            "No.of Projects": "filter-input-50",
            "Lofar Observing Time (Hrs)": "filter-input-75",
            "Lofar Processing Time (Hrs)": "filter-input-75",
            "Lofar LTA Resources(TB)": "filter-input-75",
            "Lofar Support (Hrs)": "filter-input-50",
            "Long Term Projects": "filter-input-50",
            "No.of Regular Projects": "filter-input-50",
            "Lofar Observing Time Commissioning (Hrs)": "filter-input-75",
            "Lofar Observing Time Prio A (Hrs)": "filter-input-75",
            "Lofar Observing Time Prio B (Hrs)": "filter-input-75"
        }];
        this.pageUpdated = true;

        this.setToggleBySorting();
        //this.defaultSortColumn = [{ id: "Start Date", desc: true }];
        this.toggleBySorting = this.toggleBySorting.bind(this);
        this.setToggleBySorting = this.setToggleBySorting.bind(this);
    }
    getUnitConvertedQuotaValue(cycle, cycleQuota, resourceName,cycleResources) {
        const quota = _.find(cycleQuota, { 'cycle_id': cycle.name, 'resource_type_id': resourceName });
        const  unitQuantity = cycleResources.find(i => i.name === resourceName).quantity_value;
       
        return UnitConversion.getUIResourceUnit(unitQuantity, quota ? quota.value : 0);
    }
    getCycles(cycles = [], cycleQuota,cycleResources) {
        const promises = [];
        cycles.map(cycle => promises.push(CycleService.getProjectsByCycle(cycle.name)));
        Promise.all(promises).then(responses => {
            const results = cycles;
            results.map(async (cycle, index) => {
                const projects = responses[index];
                const regularProjects = projects.filter(project => this.projectCategory.includes(project.project_category_value));
                const longterm = projects.filter(project => this.periodCategory.includes(project.period_category_value));
                cycle.duration = UnitConversion.getUIResourceUnit('days', cycle.duration);
                cycle.totalProjects = cycle.projects ? cycle.projects.length : 0;
                cycle.id = cycle.name;
                cycle.regularProjects = regularProjects.length;
                cycle.longterm = longterm.length;
                cycle.observingTime = this.getUnitConvertedQuotaValue(cycle, cycleQuota, 'LOFAR Observing Time',cycleResources);
                cycle.processingTime = this.getUnitConvertedQuotaValue(cycle, cycleQuota, 'CEP Processing Time',cycleResources);
                cycle.ltaResources = this.getUnitConvertedQuotaValue(cycle, cycleQuota, 'LTA Storage',cycleResources);
                cycle.support = this.getUnitConvertedQuotaValue(cycle, cycleQuota, 'LOFAR Support Time',cycleResources);
                cycle.observingTimeDDT = this.getUnitConvertedQuotaValue(cycle, cycleQuota, 'LOFAR Observing Time Commissioning',cycleResources);
                cycle.observingTimePrioA = this.getUnitConvertedQuotaValue(cycle, cycleQuota, 'LOFAR Observing Time prio A',cycleResources);
                cycle.observingTimePrioB = this.getUnitConvertedQuotaValue(cycle, cycleQuota, 'LOFAR Observing Time prio B',cycleResources);
                cycle['actionpath'] = `/cycle/view/${cycle.id}`;
                return cycle;
            });
            this.setState({
                cyclelist: results,
                isLoading: false
            });
        });
    }

    async componentDidMount() {
        this.pageUpdated = true;
        const promisesForAllCycleQuotasAndResources = [CycleService.getAllCycleQuotas(), CycleService.getResources()]
        const permission = await AuthUtil.getUserPermissionByModule('cycle');
        this.setState({userrole: permission});
        Promise.all(promisesForAllCycleQuotasAndResources).then( responses => {
            const cycleQuota = responses[0];                     // result from  getAllCycleQuotas
            const cycleResources = responses[1];
            this.setState({ resources: cycleResources });        // result from  getResources. 
                                                                 // Note that the state is not immediatly set, so it's better to pass them to the next function if we do need instant access.
            CycleService.getAllCycles().then(cyclelist => {
                this.getCycles(cyclelist, cycleQuota, cycleResources)
            });
        });
        this.setToggleBySorting();
    }

    setToggleBySorting() {   
        let sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });        
        if(sortData && sortData.length>0) {
            if(Object.prototype.toString.call(sortData) === '[object Array]'){
                this.defaultSortColumn = sortData;
            }   else {
                this.defaultSortColumn = [{...sortData}];
            }
        }   else {
            this.defaultSortColumn =  [{ id: "Start Date", desc: true }];
        }
        this.defaultSortColumn = this.defaultSortColumn || [];
        UtilService.localStore({ type: 'set', key: 'cycleSortData', value: this.defaultSortColumn });
    }

    toggleBySorting(sortData) { 
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [{...sortData}] });
    }

    render() {
        const  {cycle}  = this.state.userrole;
        return (
            <>
            {cycle &&
                <>
                    { /*<div className="p-grid">
                        <div className="p-col-10 p-lg-10 p-md-10">
                            <h2>Cycle - List </h2>
                        </div>
                        <div className="p-col-2 p-lg-2 p-md-2">
                            <Link to={{ pathname: '/cycle/create'}} title="Add New Cycle" style={{float: "right"}}>
                                <i className="fa fa-plus-square" style={{marginTop: "10px"}}></i>
                            </Link>
                        </div>
                    </div> */}
                    {/*
                        * Call View table to show table data, the parameters are,
                        data - Pass API data
                        defaultcolumns - This colum will be populate by default in table with header mentioned
                        showaction - {true/false} -> to show the action column
                        paths - specify the path for navigation - Table will set "id" value for each row in action button
                    */}
                    <PageHeader location={this.props.location} title={'Cycle - List'} 
                                actions={[
                                    { icon: 'fa-plus-square', 
                                        title: cycle.create ?'Click to Add Cycle': `Don't have permission to add new Cycle`,
                                        disabled: cycle.create?!cycle.create:true,  
                                        props: { pathname: '/cycle/create' } }]} />
                    {/*
                        * Call View table to show table data, the parameters are,
                        data - Pass API data
                        defaultcolumns - This colum will be populate by default in table with header mentioned
                        showaction - {true/false} -> to show the action column
                        paths - specify the path for navigation - Table will set "id" value for each row in action button
                    */}

                    {this.state.isLoading ? <AppLoader /> : (this.state.cyclelist && this.state.cyclelist.length) ?

                        <ViewTable
                            data={this.state.cyclelist}
                            defaultcolumns={this.defaultcolumns}
                            optionalcolumns={this.optionalcolumns}
                            columnclassname={this.columnclassname}
                            defaultSortColumn={this.defaultSortColumn}
                            columnOrders={this.columnOrder}
                            showaction={true}
                            paths={this.state.paths}
                            tablename={this.lsTableName}
                            toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                            lsKeySortColumn={this.lsKeySortColumn}
                            descendingColumn={this.descendingColumn}
                            pageUpdated={this.pageUpdated}
                            storeFilter={true}
                        /> : <></>
                    }
                    
                </>}
            </>
        )
    }
}

export default CycleList