//import "babel-polyfill";
import React from 'react';
import ReactDOM,  {unmountComponentAtNode} from 'react-dom';
import { BrowserRouter as Router } from 'react-router-dom';
import 'regenerator-runtime/runtime'
//import { MemoryRouter } from 'react-router-dom';
import { render, cleanup, fireEvent } from '@testing-library/react';
import mockConsole from "jest-mock-console";
import CycleList from './list';
import CycleServiceMock from '../../__mocks__/cycle.service.data';
import CycleService from '../../services/cycle.service';
import { act} from "react-dom/test-utils";
import AuthServiceMock from '../../__mocks__/auth.service.data';
import AuthUtil from '../../utils/auth.util';

let allCyclesSpy, resourcesSpy, allCycleQuotasSpy, projectsByCycleSpy, authSpy;
let container = null;
let restoreConsole;

beforeEach(() => { 
    restoreConsole = mockConsole();
    container = document.createElement("div");
    document.body.appendChild(container);
    setMockSpy();
  });

  afterEach(() => {
    restoreConsole();
    // cleanup on exiting
    clearMockSpy();
    cleanup();
    unmountComponentAtNode(container);
    container.remove();
    container = null;
  });

    const setMockSpy = (async() => {   
        authSpy = jest.spyOn(AuthUtil, 'getUserPermissionByModule');
        await authSpy.mockImplementation((a) => {
           return Promise.resolve({cycle:AuthServiceMock.cycle})
        })
        resourcesSpy = jest.spyOn(CycleService, 'getResources');
        await resourcesSpy.mockImplementation(() => {
            return Promise.resolve(CycleServiceMock.getResources);
        });   
        allCyclesSpy = jest.spyOn(CycleService, 'getAllCycles');
        await allCyclesSpy.mockImplementation(() => {
            return Promise.resolve(CycleServiceMock.getAllCycles);
        });
        allCycleQuotasSpy = jest.spyOn(CycleService, 'getAllCycleQuotas');
        await allCycleQuotasSpy.mockImplementation(() => {
            return Promise.resolve(CycleServiceMock.getAllCycleQuotas);
        });
        projectsByCycleSpy = jest.spyOn(CycleService, 'getProjectsByCycle');
        await projectsByCycleSpy.mockImplementation((id) => {
            return Promise.resolve(CycleServiceMock.getProjectsByCycle);
        }); 
    });

    const clearMockSpy = (async() => {
        authSpy.mockRestore();
        resourcesSpy.mockRestore();
        allCyclesSpy.mockRestore();
        allCycleQuotasSpy.mockRestore();
        projectsByCycleSpy.mockRestore();
    })

    it("renders without crashing", () =>{
        act(() => {
        ReactDOM.render(<Router><CycleList location={{pathname: '/cycle'}} /></Router>, container);
        });
    });

    it('renders Cycle - List Page in View Table and displays action buttons', async () => {
        let content;
       // expect(tree).toMatchSnapshot();
        await act(async () => {
            content = await render(<Router><CycleList location={{pathname: '/cycle'}} /></Router>);
        });
        await new Promise((r) => setTimeout(r, 500));
        expect(content.queryByTitle("Click to Add Cycle")).toBeInTheDocument();
        expect(content.queryByTestId('viewtable')).not.toBe(null) ;
    });

    //const flushPromises = () => new Promise(setImmediate);

    it('render table in the cycle list with data', async () => {
        let content;
        await act(async () => {
            content =await render(<Router><CycleList location={{pathname: '/cycle'}} /></Router>, container);
        });

        const element = content.queryByTestId('viewtable');
        expect(element).toBeInTheDocument();
        expect(content.queryAllByTestId('tablerow').length).toBe(4);
        expect(content.queryAllByRole('tablehead').length).toBe(8);
        
    })
 
    it('render cycleId - cycle name conversion', async () => {
        let content;
        await act(async () => {
            content =await render(<Router><CycleList location={{pathname: '/cycle'}} /></Router>, container);
        });
        expect(content.queryAllByTestId('tablerow')[1].innerHTML.includes('Cycle 16')).toBeTruthy();
    });

    it('render observing time in hours', async () => {
        let content;
        await act(async () => {
            content =await render(<Router><CycleList location={{pathname: '/cycle'}} /></Router>, container);
        });
        const observing_time = Math.floor(Number(CycleServiceMock.getAllCycleQuotas[0].value) / 3600);
        expect(content.queryAllByTestId('tablerow')[1].innerHTML.includes(observing_time)).toBeTruthy();
    });

    it('toggle columns in table', async () => {
        let content;
        await act(async () => {
            content =await render(<Router><CycleList location={{pathname: '/cycle'}} /></Router>, container);
        });
        const columnSelector = content.queryByTitle("Toggle Columns");
        // Default visible columns checked
        expect(content.queryAllByRole('tablehead').length).toBe(8);
        // Click column selector icon to open the column selection popup
        await act( async() => {
            fireEvent.click(columnSelector.children[0]);
            await new Promise((r) => setTimeout(r, 200));
        });
        // Check if popup is opened by checking the content in the popup
        expect(content.getAllByText("Select column(s) to view").length).toBe(1);
        expect(content.getAllByTitle("Toggle Column Visible").length).toBe(16);
        // Select Duration column and verify if the tablehead count is increased
        await act( async() => {
            fireEvent.click(content.getAllByTitle("Toggle Column Visible")[4])
            await new Promise((r) => setTimeout(r, 200));
        });
        expect(content.queryAllByRole('tablehead').length).toBe(9);
        // Unselect Action column and verify if the tablehead count is decreased
        await act( async() => {
            fireEvent.click(content.getAllByTitle("Toggle Column Visible")[0])
            await new Promise((r) => setTimeout(r, 200));
        });
        expect(content.queryAllByRole('tablehead').length).toBe(8);
        // Select All and verify if all columns are displayed
        await act( async() => {
            fireEvent.click(content.getAllByTitle("Toggle All Columns Hidden")[0])
            await new Promise((r) => setTimeout(r, 200));
        });
        expect(content.queryAllByRole('tablehead').length).toBe(15);
    });
 