import React from 'react';
// import ReactDOM,  {unmountComponentAtNode} from 'react-dom';
import { BrowserRouter as Router } from 'react-router-dom';
// import { act } from 'react-dom/test-utils';
import { cleanup, render, fireEvent} from '@testing-library/react';
import mockConsole from "jest-mock-console";
import ProjectList from './list';
import ProjectServiceMock from '../../__mocks__/project.service.data';
import ProjectService from '../../services/project.service';
import 'regenerator-runtime/runtime'
import AuthUtil from '../../utils/auth.util';
import SUServiceMock from '../../__mocks__/scheduleunit.service.data';

let container = null;
let authSpy, projectListSpy, clusterSpy, fileSystemSpy, projectArchiveLocationSpy, projectStateSpy, rolesSpy, friendsSpy, resourceTypeSpy;
let restoreConsole;

beforeEach(() =>{
    restoreConsole = mockConsole();
    setMockSpy();
});

afterEach(() =>{
    restoreConsole();
    clearMockSpy();
    cleanup();
})

const setMockSpy = (() => {
    authSpy = jest.spyOn(AuthUtil, 'getUserPermissionByModule');
    authSpy.mockImplementation((a) => {
        return Promise.resolve({project:AuthServiceMock.project})
    })
    projectListSpy = jest.spyOn(ProjectService, 'getProjectList');
    projectListSpy.mockImplementation((a) => {
        return Promise.resolve(ProjectServiceMock.projectWithQuota)
    })
    clusterSpy = jest.spyOn(ProjectService, 'getCluster');
    clusterSpy.mockImplementation(() =>{
        return Promise.resolve(ProjectServiceMock.cluster)
    })
    fileSystemSpy = jest.spyOn(ProjectService, 'getFileSystem');
    fileSystemSpy.mockImplementation(() => {
        return Promise.resolve(ProjectServiceMock.fileSystem)
    })
    projectArchiveLocationSpy = jest.spyOn(ProjectService, 'getAllProjectQuotaArchiveLocation');
    projectArchiveLocationSpy.mockImplementation(() => {
        return Promise.resolve(ProjectServiceMock.projectArchiveLocation)
    })
    projectStateSpy = jest.spyOn(ProjectService, 'getProjectStates');
    projectStateSpy.mockImplementation(()=> {
        return Promise.resolve(ProjectServiceMock.projectState)
    })
    friendsSpy= jest.spyOn(ProjectService, 'getFriendsOfProject');
    friendsSpy.mockImplementation((name) => {
        return Promise.resolve([])
    })
    resourceTypeSpy = jest.spyOn(ProjectService, 'getResources');
    resourceTypeSpy.mockImplementation(() => {
        let resourceType = {
            result: ProjectServiceMock.resources
        }
        return Promise.resolve(ProjectServiceMock.resourceType)
    })
    rolesSpy = jest.spyOn(ProjectService, 'getMyRoles');
    rolesSpy.mockImplementation((model) => { return Promise.resolve(SUServiceMock.getMyRoles)});   
});

const clearMockSpy = (() => {
    authSpy.mockRestore();
    projectListSpy.mockRestore();
    clusterSpy.mockRestore();
    fileSystemSpy.mockRestore();
    projectArchiveLocationSpy.mockRestore();
    projectStateSpy.mockRestore();
    rolesSpy.mockRestore();
    friendsSpy.mockRestore();
    resourceTypeSpy.mockRestore();
});

it('render project list data in table without permission to add new project, project loading status and filter', async () => {
    const content = render(<Router><ProjectList location={{pathname: "/project"}} /> </Router>);
    await new Promise((r) => setTimeout(r, 500));
    // Checks page title
    const titleElement = content.queryByText("Project - List");
    expect(titleElement).toBeInTheDocument() ;
    // If permissions not available, action buttons are disabled and title shows appropriate message
    const statusAction = content.getByTitle("Don't have permission to Update Status");
    expect(statusAction).toBeInTheDocument();
    const addAction = content.getByTitle("Don't have permission to add new Project");
    expect(addAction).toBeInTheDocument();
    // Checks if view table component loaded/rendered
    const tableElement = content.queryByTestId('viewtable');
    expect(tableElement).toBeInTheDocument();
    // Checks if all mock data listed in the table with header
    expect(content.queryAllByRole('row').length).toBe(5);
   
    //Make sure the Friend of Project is loaded and the progressbar disappear
    const topBar = content.queryAllByRole('progressbar');
    expect(topBar.length).toBe(0);
    expect(projectListSpy).toHaveBeenCalled();
    expect(friendsSpy).toHaveBeenCalled();
    expect(rolesSpy).toHaveBeenCalled();
    //test cases for filter in table
    const nameFilterELs = content.getAllByTitle('Enter few characters and press ‘Enter’ key to search');
    fireEvent.change(nameFilterELs[0], { target: { value: 'normal' } });
    expect(content.getByDisplayValue('normal')).toBeInTheDocument();
    //Actual row count will be one but there will be one more div with role="row" for "Toggle All Rows Selected" check box
    expect(content.queryAllByRole('row').length).toBe(2);
    //clear filter
    fireEvent.change(nameFilterELs[0], { target: { value: '' } });
    expect(content.getByText('high')).toBeInTheDocument();
    expect(content.getByText('low')).toBeInTheDocument();
    expect(content.getByText('normal')).toBeInTheDocument();
});
 