import React, { Component } from 'react';
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import _ from 'lodash';
import MathUtility from '../../../utils/math.utility';

/**
 * Average efficiency report component for cycle report.
 */
class CycleReportAvgEfficiency extends Component {

    constructor(props) {
        super(props);

        this.state = {}
    }

    componentDidMount() {
        const reportData = this.getTableData();
        this.setState({reportData: reportData});
        if (this.props.setExportData && reportData.length>0) {
            this.props.setExportData('AverageEfficiency', this.getTableData(true))
        }
    }

    componentDidUpdate(prevProps, prevState) {
        if (prevProps.data !== this.props.data) {
            const reportData = this.getTableData();
            this.setState({reportData: reportData});
            if (this.props.setExportData && reportData.length>0) {
                this.props.setExportData('AverageEfficiency', this.getTableData(true))
            }
        }
    }

    /**
     * Function that returns the data required for the chart with lablels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    getBarData() {
        return {
            labels: _.map(this.state.reportData, 'cycle'),
            datasets: [
                {
                    type: 'line',
                    label: 'Target',
                    borderColor: '#ff0000',
                    backgroundColor: '#ff0000',
                    borderWidth: 2,
                    fill: false,
                    data: _.map(this.state.reportData, 'target')
                },
                {
                    type: 'bar',
                    label: 'Efficiency',
                    borderColor: '#44a3ce',
                    backgroundColor: '#44a3ce',
                    barThickness: 50,
                    // barPercentage: 0.3,
                    data: _.map(this.state.reportData, 'efficiency'),
                    borderColor: 'white',
                    borderWidth: 2,
                }
            ],
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    getBarOptions() {
        return {
            // indexAxis: 'y',
            // Elements options apply to all of the options unless overridden in a dataset
            // In this case, we are setting the border of each horizontal bar to be 1px wide
            elements: {
              bar: {
                borderWidth: 1,
              },
            },
            scales: {
                y: {
                    max: 100
                }
            },
            responsive: true,
            plugins: {
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: 'Average Efficiency',
              },
            //   tooltip: {
            //     callbacks: {
            //         label: function(tooltipItem, data) {
            //             const itemData = _.find(resourceUtilization, ['type', tooltipItem.label]);
            //             return `${itemData?itemData.percent:""}% (${itemData?itemData.value:""} ${itemData?itemData.unit:""})`;
            //         }
            //     }
            //   }
            }
        };
    }

    /**
     * Function to format data list after calculations and include average if require for table or chart.
     * @param {Boolean} forTable - flag to incluce average value if the data is for table component.
     * @returns Array
     */
    getTableData(forTable) {
        const data = this.props.data;
        const reportData = [];
        _.map(data, repData=> {
            reportData.push( {cycle: repData.cycle, 
                    efficiency: repData.average_efficiency.efficiency?repData.average_efficiency.efficiency*100:null, 
                    target: repData.average_efficiency.target?repData.average_efficiency.target*100:null} );
        });
        if (forTable && reportData.length > 0) {
            reportData.push({cycle: 'Average',
                            efficiency: MathUtility.getAverage(_.map(reportData, 'efficiency')),
                            target: MathUtility.getAverage(_.map(reportData, 'target')) });
        }
        return reportData;
    }

    render() {
        return (
            <>
                <Bar data={this.getBarData()} options={this.getBarOptions()} />
                <DataTable value={this.getTableData(true)} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                    <Column field="cycle" header="Cycle code"></Column>
                    <Column field="efficiency" header="Efficiency (%)"></Column>
                    <Column field="target" header="Target (%)"></Column>
                </DataTable>
            </>
        );
    }
}

export default CycleReportAvgEfficiency;