import React,{ Component } from "react";
import { Bar } from 'react-chartjs-2';
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import UnitConverter from "../../../utils/unit.converter";
import MathUtility from "../../../utils/math.utility";

/**
 * Component to show data ingested per category in cycle(s).
 */
class CycleCategoryWiseData extends Component{

    constructor(props){
        super(props);

        this.state= {
            reportData:[]
        };
    }

    getBarData() {
        let reportData = this.getReportData(false);
        if (this.props.setExportData && reportData.length>0) {
            this.props.setExportData('DataIngestedPerCategory', reportData);
        }
        return {
            labels: _.map(reportData, 'cycle'),
            datasets:[
                
                {   label: 'Raw IF', 
                    data:_.map(reportData, 'rawIF'),
                    backgroundColor: '#4472C4',
                    barThickness: 75,
                },
                {   label: 'Raw BF', 
                    data:_.map(reportData, 'rawBF'),
                    backgroundColor: '#ED7D31',
                    barThickness: 75,
                            
                },
                // {   label: 'Raw TBB', 
                //     data:_.map(reportData, 'rawTBB'),
                //     backgroundColor: '#A5A5A5',
                //     barThickness: 75,       
                // },
                {   label: 'Preproc IF', 
                    data:_.map(reportData, 'preprocIF'),
                    backgroundColor: '#FFC000',
                    barThickness: 75,       
                },
                {   label: 'Pulp BF', 
                    data:_.map(reportData, 'pulpBF'),
                    backgroundColor: '#44a3ce',
                    barThickness: 75,       
                },
                // {   label: 'Dynspec BF', 
                //     data:_.map(reportData, 'dynspecBF'),
                //     backgroundColor:  '#4caf50',
                //     barThickness: 75,       
                // }
            ]
        };
    }

    getBarOptions() {
        return {
            indexAxis: 'x',
            elements: {
              bar: {
                borderWidth: 0,
              },
            },
            scales: {
                x: {
                    stacked: true,
                    categoryPercentage: 1.0,
                    barPercentage: 1.0
                },
                y: {
                    stacked: true,
                    // max: 5000,
                }
            },
            responsive: true,
            plugins: {
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: 'Ingested Data per Type (TB)',
              }
            }
        };
    }

    getReportData(isTableData) {
        let reportData = [];
        const dataConversionFactor = UnitConverter.resourceUnitMap["bytes"].conversionFactor;
        this.props.data.map(repData => {
            let categoryData = {}
            const cycleData = repData.data_ingested_per_site_and_category.categories;
            categoryData['cycle'] = repData.cycle;
            // TODO: Update field names and values once API is complete
            categoryData['rawIF'] = (cycleData["Interferometric Observation"].size__sum/dataConversionFactor).toFixed(2);
            categoryData['rawBF'] = (cycleData["Beamformed Observation"].size__sum/dataConversionFactor).toFixed(2);
            // categoryData['rawTBB'] = cycleData.rawTBB;
            categoryData['preprocIF'] = (cycleData["Preprocessing Pipeline"].size__sum/dataConversionFactor).toFixed(2);
            categoryData['pulpBF'] = (cycleData["Pulsar Pipeline"].size__sum/dataConversionFactor).toFixed(2);
            // categoryData['dynspecBF'] = cycleData.dynspecBF;
            reportData.push(categoryData);
            return repData;
        });

        if (isTableData) {
            reportData.push({cycle: 'Average',
                rawIF: MathUtility.getAverage(_.map(reportData, 'rawIF'), 2), 
                rawBF: MathUtility.getAverage(_.map(reportData, 'rawBF'), 2), 
                // rawTBB: MathUtility.getAverage(_.map(reportData, 'rawTBB')), 
                preprocIF: MathUtility.getAverage(_.map(reportData, 'preprocIF'), 2), 
                pulpBF: MathUtility.getAverage(_.map(reportData, 'pulpBF'), 2), 
                // dynspecBF: MathUtility.getAverage(_.map(reportData, 'dynspecBF'))
            });
        }
        return reportData;
    }

    render() {
        return(
            <React.Fragment>
                <div  id={`cycles-category-data`}
                    style={{paddingTop: "10px", paddingBottom: "10px"}}>
                    <Bar data={this.getBarData()} options={this.getBarOptions()} width="50%" height="20" />
                </div>
                
                <div className="ttd-details" id={`cycle-ttd-details`}>
                    <DataTable value={this.getReportData(true)} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em', textAlign: 'center'}}>
                        <Column field="cycle" header="Cycle Code"></Column>
                        <Column field="rawIF" header="Raw IF (TB)"></Column>
                        <Column field="rawBF" header="Raw BF (TB)"></Column>
                        {/* <Column field="rawTBB" header="Raw TBB (TB)"></Column> */}
                        <Column field="preprocIF" header="Preproc IF (TB)"></Column>
                        <Column field="pulpBF" header="Pulp BF (TB)"></Column>
                        {/* <Column field="dynspecBF" header="Dynspec BF (TB)"></Column> */}
                    </DataTable>
                </div>

            </React.Fragment>
        )
    }

}

export default CycleCategoryWiseData;