import React,{ Component } from "react";
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import MathUtility from "../../../utils/math.utility";
import UnitConverter from "../../../utils/unit.converter";

/**
 * Component to show Observations per category in a cycle.
 */
class CycleCategoryObservations extends Component{

    constructor(props){
        super(props);

        this.state= {
            reportData:[]
        };
    }

    getBarData() {
        let reportData = this.getReportData(false);
        return {
            labels: _.map(reportData, 'cycle'),
            datasets:[
                
                {   label: 'Prio A', 
                    data:_.map(reportData, 'prio_A'),
                    backgroundColor: '#4472C4',
                    barThickness: 75,
                },
                {   label: 'Prio B', 
                    data:_.map(reportData, 'prio_B'),
                    backgroundColor: '#ED7D31',
                    barThickness: 75,
                            
                },
                {   label: 'DDT_Com_Rep', 
                    data:_.map(reportData, 'ddt_com_rep'),
                    backgroundColor: '#A5A5A5',
                    barThickness: 75,       
                },
                {   label: 'TMSS Com', 
                    data:_.map(reportData, 'tmss_com'),
                    backgroundColor: '#FFC000',
                    barThickness: 75,       
                },
                {   label: 'System Unavailability/O_com', 
                    data:_.map(reportData, 'unavailable'),
                    borderColor: '#ff0000',
                    backgroundColor: '#ff0000',
                    barThickness: 75,       
                },
                {   label: 'Failed', 
                    data:_.map(reportData, 'failed'),
                    backgroundColor: '#44a3ce',
                    barThickness: 75,       
                },
                {   label: 'System/Idle', 
                    data:_.map(reportData, 'idle'),
                    backgroundColor:  '#4caf50',
                    barThickness: 75,       
                }
            ]
        };
    }

    getBarOptions() {
        return {
            indexAxis: 'x',
            elements: {
              bar: {
                borderWidth: 0,
              },
            },
            scales: {
                x: {
                    stacked: true,
                    categoryPercentage: 1.0,
                    barPercentage: 1.0
                },
                y: {
                    stacked: true,
                    // max: 5000,
                }
            },
            responsive: true,
            plugins: {
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: 'Observation Hours per Category',
              }
            }
        };
    }

    getReportData(isTableData) {
        let reportData = [];
        const timeConversionFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
        this.props.data.map(repData => {
            let cycleData = {}
            cycleData['cycle'] = repData.cycle;
            cycleData['prio_A'] = (repData.observation_durations_per_category.total_duration_successful_A/timeConversionFactor).toFixed(2);
            cycleData['prio_B'] = (repData.observation_durations_per_category.total_duration_successful_B/timeConversionFactor).toFixed(2);
            cycleData['ddt_com_rep'] = (repData.observation_durations_per_category["DDT Com Rep"]/timeConversionFactor).toFixed(2);
            cycleData['unavailable'] = (repData.observation_durations_per_category["System Unavailability"]/timeConversionFactor).toFixed(2);
            cycleData['tmss_com'] = (repData.observation_durations_per_category["TMSS Com"]/timeConversionFactor).toFixed(2);
            cycleData['failed'] = (repData.observation_durations_per_category.total_duration_failed/timeConversionFactor).toFixed(2);
            cycleData['idle'] = (repData.observation_durations_per_category.total_duration_idle/timeConversionFactor).toFixed(2);
            reportData.push(cycleData);
        });

        if (isTableData && reportData.length>0) {
            reportData.push({cycle: 'Average',
                prio_A: MathUtility.getAverage(_.map(reportData, 'prio_A')), 
                prio_B: MathUtility.getAverage(_.map(reportData, 'prio_B')), 
                ddt_com_rep: MathUtility.getAverage(_.map(reportData, 'ddt_com_rep')),
                unavailable: MathUtility.getAverage(_.map(reportData, 'unavailable')), 
                failed: MathUtility.getAverage(_.map(reportData, 'failed')), 
                idle: MathUtility.getAverage(_.map(reportData, 'idle'))
            });
            if (this.props.setExportData) {
                this.props.setExportData('CategoryObservation', reportData);
            }
        }
        return reportData;
    }

    render() {
        return(
            <React.Fragment>
                <div  id={`cycles-category-time`}
                    style={{paddingTop: "10px", paddingBottom: "10px"}}>
                    <Bar data={this.getBarData()} options={this.getBarOptions()} width="50%" height="20" />
                </div>
                
                <div className="ttd-details" id={`cycle-ttd-details`}>
                    <DataTable value={this.getReportData(true)} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em', textAlign: 'center'}}>
                        <Column field="cycle" header="Cycle Code"></Column>
                        <Column field="prio_A" header="Prio A (hrs)"></Column>
                        <Column field="prio_B" header="Prio B (hrs)"></Column>
                        <Column field="ddt_com_rep" header="DDT Com Rep (hrs)"></Column>
                        <Column field="tmss_comm" header="TMSS Comm (hrs)"></Column>
                        <Column field="unavailable" header="System Unavailability (hrs)"></Column>
                        <Column field="failed" header="Failed (hrs)"></Column>
                        <Column field="idle" header="System/Idle (hrs)"></Column>
                    </DataTable>
                </div>

            </React.Fragment>
        )
    }

}

export default CycleCategoryObservations;