import React, { Component } from 'react';
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import { Dropdown } from 'primereact/dropdown';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import _ from 'lodash';
import moment from 'moment';
import MathUtility from '../../../utils/math.utility';

/**
 * Weekly efficiency report component for cycle report.
 */
class CycleWeeklyEfficiency extends Component {

    constructor(props) {

        super(props);

        this.state = {
            // reportData: this.getReportData(),
            // selectedCycle: this.cycles[0]
        }
    }

    componentDidMount() {
        this.setState({
            reportData: this.getReportData(),
            selectedCycle: this.cycles[0]
        });
    }

    componentDidUpdate(prevProps, prevState) {
        if (prevProps.data !== this.props.data) {
            this.setState({
                reportData: this.getReportData(),
                selectedCycle: this.cycles[0]
            });
        }
    }

    /**
     * Function that returns the data required for the chart with lablels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    getBarData() {
        const selectedCycle = this.state.selectedCycle || this.cycles[0];
        const barData = _.filter(this.state.reportData.chartData, ['cycle', selectedCycle]) ;
        return {
            labels: _.map(barData, 'Week Index'),
            datasets: [
                {
                    type: 'bar',
                    label: 'Efficiency %',
                    borderColor: '#44a3ce',
                    backgroundColor: '#44a3ce',
                    borderWidth: 2,
                    barThickness: 25,
                    // barPercentage: 0.5,
                    data: _.map(barData, 'efficiency'),
                    borderColor: 'white',
                    borderWidth: 2,
                }
            ],
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    getBarOptions() {
        return {
            elements: {
              bar: {
                borderWidth: 1,
              },
            },
            scales: {
                y: {
                    max: 100
                }
            },
            responsive: true,
            plugins: {
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: (this.state.selectedCycle || this.cycles[0] || '') + ' Weekly Efficiency (%)',
              },
            //   tooltip: {
            //     callbacks: {
            //         label: function(tooltipItem, data) {
            //             const itemData = _.find(resourceUtilization, ['type', tooltipItem.label]);
            //             return `${itemData?itemData.percent:""}% (${itemData?itemData.value:""} ${itemData?itemData.unit:""})`;
            //         }
            //     }
            //   }
            }
        };
    }

    /**
     * Function to format data list after calculations and include average if require for table or chart.
     * @param {Boolean} forTable - flag to incluce average value if the data is for table component.
     * @returns Array
     */
    getReportData(forTable) {
        const data = this.props.data;
        let reportData = {chartData: [], tableData: []};
        _.map(data, repData=> {
            const weekly_efficiency = repData.weekly_efficiency;
            let index = 1;
            for (const weekData of weekly_efficiency.weeks) {
                reportData.chartData.push({   cycle: repData.cycle,
                                    weekDate: weekData.week,
                                    weekNumber: moment(weekData.week).week(),
                                    'Week Index': 'Week - ' + index,
                                    efficiency: parseInt((weekData.efficiency * 100).toFixed(0))
                                })
                index++;
            }
        });
        this.cycles = _.keys(_.groupBy(reportData.chartData, 'cycle'));
        // if (forTable) {
            const weekGroupData = _.groupBy(reportData.chartData, 'Week Index');
            let groupedTableData = []
            for (const week of _.keys(weekGroupData)) {
                const weekData = weekGroupData[week];
                let groupData = {'Week Index': week };
                for (const cycle of weekData) {
                    groupData[cycle.cycle + " Efficiency %"] = cycle.efficiency;
                }
                reportData.tableData.push(groupData);
            }
            // reportData = groupedTableData;
            if (reportData.tableData.length > 0) {
                let averageData = {'Week Index': 'Average'};
                for (const column of (_.keys(reportData.tableData[0]))) {
                    if (column !== 'Week Index') {
                        averageData[column] = MathUtility.getAverage(_.map(reportData.tableData, column));
                    }
                }
                reportData.tableData.push(averageData);
            }
        // }
        if (this.props.setExportData && reportData.tableData.length > 0) {
            this.props.setExportData('WeeklyEfficiency', reportData.tableData);
        }
        return reportData;
    }

    renderReportTable() {
        const reportTableData = this.state.reportData.tableData;
        const reportColumns = _.keys(reportTableData[0]).map((col,i) => {
            return <Column key={col} field={col} header={col} />;
        });
        return (
            <DataTable value={reportTableData} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                { reportColumns }
            </DataTable>
        );
    }
    
    render() {
        return (
            <>
                <Dropdown value={this.state.selectedCycle} options={this.cycles} 
                    onChange={e => this.setState({selectedCycle:  e.value})}
                    style={{width: '200px', marginLeft: '20px'}} />
                {this.state.reportData &&
                <>
                    <Bar data={this.getBarData()} options={this.getBarOptions()} />
                    { this.renderReportTable() }
                </>
                }
            </>
        );
    }
}

export default CycleWeeklyEfficiency;