import React, { Component } from 'react';
import _ from 'lodash';
import moment from 'moment';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import { Dropdown } from 'primereact/dropdown';

import "flatpickr/dist/flatpickr.css";
import Flatpickr from "react-flatpickr";
import confirmDatePlugin from "flatpickr/dist/plugins/confirmDate/confirmDate";

import { CustomDialog } from '../../layout/components/CustomDialog';
import { appGrowl } from '../../layout/components/AppGrowl';
import AppLoader from "../../layout/components/AppLoader";
import ViewTable from '../../components/ViewTable';
import PageHeader from '../../layout/components/PageHeader';

import UnitService from '../../utils/unit.converter';
import UIConstants from '../../utils/ui.constants';
import Validator from '../../utils/validator';

import ReservationService from '../../services/reservation.service'; 
import CycleService from '../../services/cycle.service';
import UtilService from '../../services/util.service';
import AuthStore from '../../authenticate/auth.store';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import Websocket from 'react-websocket';

export class ReservationList extends Component{
    lsKeySortColumn = "ReservationListSortData";
    constructor(props){
        super(props);
        this.state = {
            validFields: {},
            fStartTime: '',   // Filter Start time
            fEndTime: '',     // Filter End Time
            reservationsList: [],
            cycle: '',
            errors: {},
            dialog: {},
            defaultcolumns: [{
                name: {name:"System Id"},
                description: {name:"Description"},
                start_time: {
                    name: "Start Time",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                stop_time: {
                    name: "End Time",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                duration:{
                    name:"Duration (Days HH:mm:ss)",
                    format:UIConstants.CALENDAR_TIME_FORMAT,
                    filter: "durationWithDaysMinMax",
                },
                reservation_type: {
                    name:"Reservation Type",
                    filter:"select"
                },
                subject: {
                    name:"Subject",
                    filter:"select"
                },
                planned: {
                    name: "Planned",
                    filter:"switch"
                },
                stations:{
                    name: "Stations",
                    filter:"multiselect"
                },
                fixed_time:  {
                    name: "FixedTime",
                    filter:"switch"
                },
                dynamic: {
                    name: "Dynamic",
                    filter:"switch"
                },
                project_exclusive: {
                    name: "Fixed project",
                    filter:"switch"
                },
                project_id: {
                    name: "Project",
                    filter:"select"
                },
                expert: {
                    name: "Expert",
                    filter:"switch"
                },
                hba_rfi: {
                    name: "HBA-RFI",
                    filter: 'switch'
                },
                lba_rfi: {
                    name: "LBA-RFI",
                    filter: 'switch'
                },
                created_at: {
                    name: "Created at",
                    filter: "flatpickrDateRange",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                updated_at: {
                    name: "Updated at",
                    filter: "flatpickrDateRange",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                actionpath: "actionpath"
            }],
            optionalcolumns:  [{
                id: {name:"ID"},
                
            }],
            columnclassname: [{
                "ID": "filter-input-50",
                "System Id": "filter-input-100",
                "Description": "filter-input-100",
                "Duration (Days HH:mm:ss)":"filter-input-150",
                "Reservation type":"filter-input-100",
                "Subject":"filter-input-75",
                "Planned":"filter-input-50",
                "Stations":"filter-input-150,multi-select",
                "FixedTime":"filter-input-50",
                "Dynamic":"filter-input-50",
                "Fixed project":"filter-input-50",
                "Expert":"filter-input-50",
                "HBA-RFI":"filter-input-50",
                "LBA-RFI":"filter-input-50",
                "Start Time":"filter-input-150",
                "End Time":"filter-input-150",
                "Created at":"filter-input-150",
                "Updated at":"filter-input-150",    
            }],
            columnOrders: [
                "ID",
                "System Id",
                "Description",
                "Start Time",
                "End Time",
                "Duration (Days HH:mm:ss)",
                "Reservation Type",
                "Subject",
                "Planned",
                "Stations",
                "FixedTime",
                "Dynamic",
                "Fixed project",
                "Project",
                "Expert",
                "HBA-RFI",
                "LBA-RFI",
                "Created at",
                "Updated at"
            ],
            defaultSortColumn: [{id: "System Id", desc: false}],
            isLoading: true,
            cycleList: [],
            userrole: AuthStore.getState()
        }
        this.reservationTypeOptionList = [];
        this.subjectsOptionList = [];
        this.stationList = [];
       // let optionsList = new Set();
        this.filterQry = '';
        this.orderBy = '';
        this.limit = 10;
        this.offset =  0;
        this.currentPageSize = 10;
        this.tableInstance = null;
        this.reservations= [];
        this.cycleList= [];
        this.selectedRows = [];
        this.totalPage = 0;
        this.pageUpdated = true;
        this.onRowSelection = this.onRowSelection.bind(this);
        this.confirmDeleteReservations = this.confirmDeleteReservations.bind(this);
        this.deleteReservations = this.deleteReservations.bind(this);
        this.closeDialog = this.closeDialog.bind(this);
        this.getReservationDialogContent = this.getReservationDialogContent.bind(this);
        this.closeList = this.closeList.bind(this);
        this.fetchTableData = this.fetchTableData.bind(this);
        this.getFilterOptions = this.getFilterOptions.bind(this);
        this.resetStartDateTime = this.resetStartDateTime.bind(this);
        this.resetEndDateTime = this.resetEndDateTime.bind(this);
        this.clearAllFilter = this.clearAllFilter.bind(this);
        this.seTableInstanceRef = this.seTableInstanceRef.bind(this);
        this.setDateRange = this.setDateRange.bind(this);
        this.onConnect = this.onConnect.bind(this);
        this.handleData = this.handleData.bind(this);
        this.refreshReservationList = this.refreshReservationList.bind(this);
        this.updateReservation = this.updateReservation.bind(this);
    }
    
    async componentDidMount() {
        this. setToggleBySorting();
        await this.getFilterColumns();
        this.cycleList = await CycleService.getAllCycles();
        this.setLocalFilters();
        await this.setState({
            isLoading: false,
            cycleList: this.cycleList,
            reservationsList: this.reservations,
        });
    }

    /**
     * Set Top filters when the state changed or while init the page
     */
    setLocalFilters() {
        let filters = UtilService.localStore({ type: 'get', key: "reservation_list"});
        if (filters) {
            let filter = _.find(filters, {'id': 'CycleId'});
            if (filter) {
                this.setState({cycle: filter.value});
            }
            filter = _.find(filters, {'id': 'Start Time'});
            if (filter) {
                const values = filter.value;
                this.setState({fStartTime: values[0]});
            }   else {
                this.setState({fStartTime: '', cycle: ''});
            }
            filter = _.find(filters, {'id': 'End Time'});
            if (filter) {
                const values = filter.value;
                this.setState({fEndTime:  values.length === 2 ? values[1] : values[0]});
            }   else {
                this.setState({fEndTime: '', cycle: ''});
            }
        }
        let filterStartTime = _.find(filters, function(filter){
            return (filter.id === 'Start Time' && filter.value.length>0);
        });
        let filterEndTime = _.find(filters, function(filter){
            return (filter.id === 'End Time' && filter.value.length>0)
        });
        if (!filterStartTime && !filterEndTime) {
            this.setState({cycle: ''});
        }
    }

    toggleBySorting=(sortData) => {       
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: sortData });
    }

    setToggleBySorting() {      
        let sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        if(sortData){
            if(Object.prototype.toString.call(sortData) === '[object Array]'){
                this.defaultSortColumn = sortData;
            }
            else{
                this.defaultSortColumn = [{...sortData}];
            }
        }
        this.defaultSortColumn = this.defaultSortColumn || [];
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [...this.defaultSortColumn] });       
    }
	
    mergeResourceWithReservation ( reservation, params) {
        if( params ){
            Object.keys(params).map((key, i) => (
                ['name', 'description'].indexOf(key)<0? reservation[key]= params[key] : ''      
              ));
        }
        return reservation;
    }

    /**
     * Filter reservation based on cycle filter selected
     * Here to search use Cycle.start time as Reservation Start time and Cycle.stop time as Reservation End time
     */
    async filterTableData(cycleValues) {
        let filters = UtilService.localStore({ type: 'get', key: "reservation_list"});
        _.remove(filters, function(filter) {
            return filter.id === 'Cycle' || filter.id === 'CycleId' || filter.id === 'Start Time' || filter.id === 'End Time';
        });
        if (!cycleValues) {
            UtilService.localStore({ type: 'set', key: 'reservation_list', value: filters});                       
            await this.setState({cycle: '',fStartTime:'', fEndTime: ''});
            this.setTableProperty(filters, 'Filter');
            await this.fetchTableData(null);
        } else {
            const filterCycleList = _.filter(this.cycleList, function(o) { return o.name === cycleValues });
            if (filterCycleList) {
                let cycle = filterCycleList[0];
                this.setState({fStartTime: cycle['start'], fEndTime: cycle['stop']});
                let cycleFilter = [{'id': 'Cycle', value:[cycle['start'], cycle['stop']]}];
                cycleFilter.push({'id': 'CycleId', value:cycle['name']});
                cycleFilter.push({'id': 'Start Time', value:[cycle['start'], '']});
                cycleFilter.push({'id': 'End Time', value:['', cycle['stop']]});
                filters = [...cycleFilter, ...filters]; 
                UtilService.localStore({ type: 'set', key: 'reservation_list', value: filters});
                this.setTableProperty(filters, 'Filter');
                await this.fetchTableData(null);
            }
            await this.setState({cycle: cycleValues,});
        }
        this.pageUpdated = true;
    }

    /**
    Reset the Reserved Start Datetime filter
     */
    async resetStartDateTime() {
        await this.setState({'fStartTime': ''});
        let filters = UtilService.localStore({ type: 'get', key: "reservation_list"});
        _.remove(filters, function(filter) {
            let result =  filter.id === 'Start Time';
            return result;
        });
        UtilService.localStore({ type: 'set', key: 'reservation_list', value: filters});
        this.setTableProperty(filters, 'Filter');
        await this.fetchTableData(null);
    }

    /**
    Reset the Reserved End Datetime filter
     */
    async resetEndDateTime() {
        await this.setState({'fEndTime': ''});
        let filters = UtilService.localStore({ type: 'get', key: "reservation_list"});
        _.remove(filters, function(filter) {
            let result =  filter.id === 'End Time';
            return result;
        });
        UtilService.localStore({ type: 'set', key: 'reservation_list', value: filters});
        this.setTableProperty(filters, 'Filter');
        await this.fetchTableData(null);
    }

    /**
        - To clear all filters above the table, it will be called from ViewTable-ClearAllFilter
     */
    clearAllFilter() {        
        this.setState({cycle: '', fStartTime: '', fEndTime: ''});
    }

    /**
     * Get View Table instance
     * @param {*} instanceRef 
     */
    seTableInstanceRef(instanceRef) {
        this.tableInstance = instanceRef;
    }

    /**
     * Set View table Properties from parent component
     * @param {*} value - Value to set
     * @param {*} propsType - Property type like to set the Filter, Columns Header
     */
    setTableProperty(value, propsType) {
        if (this.tableInstance) {
            if (propsType === 'Filter') {
                this.tableInstance.state.filters = value;
            }
        }
    }

    /**
     * Set Filter: Start/End date and time. It will display the reservation which is active during the time frame
     * @param {*} type - Date Filter Name
     * @param {*} value - Date Value
     */
    async setDateRange() {
        let filters = UtilService.localStore({ type: 'get', key: "reservation_list"});
        _.remove(filters, function(filter) {
            let result =  filter.id === 'Start Time' ||  filter.id === 'End Time';
            return result;
        });
        await this.setState({cycle: ''});
        let reservedTimeFilter = [{'id': 'Start Time', value:[Validator.isEmpty(this.state.fStartTime)?'':moment(new Date(this.state.fStartTime)).format("YYYY-MM-DDTHH:mm:SS"), '']}];
        reservedTimeFilter.push({'id': 'End Time', value:['', Validator.isEmpty(this.state.fEndTime)?'':moment(new Date(this.state.fEndTime)).format("YYYY-MM-DDTHH:mm:SS")]});
        filters = [...reservedTimeFilter, ...filters]; 
        this.setTableProperty(filters, 'Filter');
        UtilService.localStore({ type: 'set', key: 'reservation_list', value: filters});
        await this.fetchTableData(null);
        this.pageUpdated = true;
    }
 
    /**
     * Set selected rows form view table
     * @param {Row} selectedRows - rows selected in view table
     */
    onRowSelection(selectedRows) {
        this.pageUpdated = false;
        this.selectedRows = selectedRows;
    }

    /**
     * Callback function to close the dialog prompted.
     */
     closeDialog() {
        this.setState({dialogVisible: false});
    }

    closeList(){
        this.props.history.length>1?this.props.history.goBack():this.props.history.push(`/su/timelineview/week`);
    }

    /**
     * Create confirmation dialog details
     */
    confirmDeleteReservations() {
        if(this.selectedRows.length === 0) {
            appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Select Reservation to delete.'});
        }   else {
            let dialog = {};
            dialog.type = "confirmation";
            dialog.header= "Confirm to Delete Reservation(s)";
            dialog.detail = "Do you want to delete the selected Reservation(s)?";
            dialog.content = this.getReservationDialogContent;
            dialog.actions = [{id: 'yes', title: 'Delete', callback: this.deleteReservations, className:'act-btn-dispose'},
            {id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog}];
            dialog.onSubmit = this.deleteReservations;
            dialog.width = '55vw';
            dialog.showIcon = false;
            this.setState({dialog: dialog, dialogVisible: true});
        }
    }

     /**
     * Prepare Reservation(s) details to show on confirmation dialog
     */
      getReservationDialogContent() {
        return  <>  
                <DataTable value={this.selectedRows} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                        <Column field="id" header="Reservation Id"></Column>
                        <Column field="name" header="Name"></Column>
                        <Column field="start_time" header="Start time"></Column>
                        <Column field="stop_time" header="End Time"></Column>
                </DataTable>
        </>
    }

    /**
     * Delete selected Reservation(s)
     */
     async deleteReservations() {
        let hasError = false;
        for(const reservation of this.selectedRows) {
            if(!await  ReservationService.deleteReservation(reservation.id)) {
                hasError = true;
            }
        }
        if(hasError){
            appGrowl.show({severity: 'error', summary: 'error', detail: 'Error while deleting Reservation(s)'});
            this.setState({dialogVisible: false});
        }   else {
            this.selectedRows = [];
            this.setState({dialogVisible: false});
            this.componentDidMount();
            appGrowl.show({severity: 'success', summary: 'Success', detail: 'Reservation(s) deleted successfully'});
        }
    }

    /**
     * Prepare API FIlter column to view table component
     * @param {*} apiFilters 
     * @param {*} columnDef 
     * @returns 
     */
    getAPIFilter(apiFilters, columnDef) {
        const defaultColKeys = Object.keys(columnDef);
        defaultColKeys.forEach(key => {
            let tmpColMap = {};
            let tempKey = key;
            tmpColMap['orgField'] = tempKey;
            tmpColMap['tmpField'] = tempKey;
            if(columnDef[key]) {
                tmpColMap['displayName'] = columnDef[key]['name'];
            }
            this.columnMap.push(tmpColMap);
            //Set Enable/Disable the Filter & SortBy in each column
            if(apiFilters.data.filters[tempKey]) {
                columnDef[key]['disableSortBy'] = !_.includes(apiFilters.data.ordering, tempKey);
                columnDef[key]['disableFilters'] = false;
                if( (tempKey !== 'start_time' && tempKey !== 'stop_time') && UIConstants.FILTER_MAP[apiFilters.data.filters[tempKey].type]) {
                    columnDef[key]['filter'] = UIConstants.FILTER_MAP[apiFilters.data.filters[tempKey].type];
                }
            }   else if (key === 'project_id' && apiFilters.data.filters['project']) {
                columnDef[key]['disableSortBy'] = !_.includes(apiFilters.data.ordering, 'project');
                columnDef[key]['disableFilters'] = false;
                columnDef[key]['filter'] = '';
            }   else if (key === 'stations' && apiFilters.data.filters['stations_any']) {
                columnDef[key]['disableSortBy'] = false;
                columnDef[key]['disableFilters'] = false;
            }   else if (key === 'duration' && apiFilters.data.filters['duration_min']) {
                columnDef[key]['disableSortBy'] = !_.includes(apiFilters.data.ordering, 'duration_min');
                columnDef[key]['disableFilters'] = false;
            }   else if (columnDef[key]['name']) {
                columnDef[key]['disableSortBy'] = true;
                columnDef[key]['disableFilters'] = true;
            }
        });
        return columnDef;
    }

    /**
     * Remove column in custom column order
     * @param {array} arrayValue 
     * @param {array} keys 
     */
    removeArrayIndex(arrayValue, keys) {
        keys.forEach(key => {
            let index = arrayValue.indexOf(key) ;
            if (index !== -1) {
                arrayValue.splice(index, 1);
            }
        });
    }
    
    /**
     * Remove column definition
     * @param {column definition array} arrayValue 
     * @param {array} keys 
     */
    removeColumns(arrayValue, keys) {
        keys.forEach(key => {
            delete arrayValue[key];
        });
    }
    /**
     * Get server side filter column details form API
     */
    async getFilterColumns() {
        const apiFilters = await ReservationService.getReservationFilterDefinition();
        this.columnMap = [];
        let tmpDefaulColumns = _.cloneDeep(this.state.defaultcolumns[0]);
        let tmpOptionalColumns = _.cloneDeep(this.state.optionalcolumns[0]);
        let tmpColumnOrders = _.cloneDeep(this.state.columnOrders);
         
        if(apiFilters) {
            this.getDropDownOptionList(apiFilters);
            tmpDefaulColumns = this.getAPIFilter(apiFilters, tmpDefaulColumns);
            tmpOptionalColumns = this.getAPIFilter(apiFilters, tmpOptionalColumns);
            await this.setState({tmpDefaulcolumns: [tmpDefaulColumns], tmpOptionalcolumns:[tmpOptionalColumns], tmpColumnOrders: tmpColumnOrders, columnMap: this.columnMap})
        }
    }

    /**
     * Get Status list frol filter
     * @param {Array} suFilters 
     */
     getDropDownOptionList(apiFilters) {
        this.getOptions('reservation_type', apiFilters);
        this.getOptions('subject', apiFilters);
        this.stationList = [];
        this.getOptions('stations_any', apiFilters);
    }

    getOptions(key, apiFilter) {
        if (apiFilter.data.filters[key]) {
            apiFilter.data.filters[key].choices.forEach(choice => {
                if (key === 'subject') {
                    this.subjectsOptionList.push(choice.value);
                }   else if(key === 'reservation_type') {
                    this.reservationTypeOptionList.push(choice.value);    
                }   else if (key === 'stations_any') {
                    this.stationList.push({name: choice.display_name, value:choice.value});
                }
            })
        }
    }

    /**
     * Fetch data from server side - while doing pagination, filtering, sorting
     * @param {Table State} Table props state 
     * @returns 
     */
    async fetchTableData(state) {
       // await this.getFilterColumns();
        await this.setLocalFilters();
        this.filterQry = '';
        this.orderBy = '';
        this.pageUpdated = true;
        this.setState({loadingStatus:true});
        let filters = UtilService.localStore({ type: 'get', key: "reservation_list"});
        const sortByValue = UtilService.localStore({ type: 'get', key: "ReservationListSortData"});
        if(filters.length > 0 ) {
            for( const filter of filters) {
                if (filter.id === 'Start Time') {
                    const values = filter.value;
                    if (values[0] && values[0] !== '') {
                        this.filterQry += 'start_time_min='+ moment(new Date(values[0])).format("YYYY-MM-DD HH:mm:SS&");
                    }
                    if (values[1] && values[1] !== '') {
                        this.filterQry += 'start_time_max='+moment(new Date(values[1])).format("YYYY-MM-DD HH:mm:ss&");
                    }
                }   else if (filter.id === 'End Time') {
                    const values = filter.value;
                    if (values[0] && values[0] !== '') {
                        this.filterQry += 'stop_time_min='+ moment(new Date(values[0])).format("YYYY-MM-DD 00:00:00&");
                    }
                    if (values[1] && values[1] !== '') {
                        this.filterQry += 'stop_time_max='+moment(new Date(values[1])).format("YYYY-MM-DD 23:59:59&");
                    }
                }   else if (filter.id === 'Project') {
                    this.filterQry += 'project='+ encodeURIComponent(filter.value) +'&' ;
                }   else if (filter.id === 'Stations') {
                    const stationFilterType = _.find(filters, {id:'stationFilterType'});
                    if(filter.value.length>0) {
                        const values = _.split(filter.value, ",");
                        for ( const value of values) {
                            if(stationFilterType && stationFilterType.value === 'All') {
                                this.filterQry += 'stations_all='+value+"&";
                            }   else {
                                this.filterQry += 'stations_any='+value+"&";
                            }
                        }
                    }
                }   else if (filter.id === 'Duration (Days HH:mm:ss)' && filter.value !== '') {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    const values = _.split(filter.value, ",");
                    if ((values.length === 0) || (values[0] === '' || values[0] === '0') && (values[1] === '' || values[1] === '0')) {
                        this.filterQry += "duration_isnull=false&";
                    }   else {
                        if (values[0].includes(":")) {
                            this.filterQry += columnDetails.orgField+"_min" +'='+UnitService.getDDDHHmmssToSecs(values[0])+'&';
                        }
                        if (values[1].includes(":")) {
                            this.filterQry += columnDetails.orgField+"_max" +'='+UnitService.getDDDHHmmssToSecs(values[1])+'&';
                        }
                    }
                }   else if (filter.id === 'durationNull') {
                    this.filterQry += "duration_isnull="+filter.value+'&';
                }    else {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    if(columnDetails) {
                        this.filterQry += columnDetails.orgField +'='+ encodeURIComponent(filter.value) +'&'
                    }
                }  
            }
        }

        let sortBy = state && state.sortBy?state.sortBy[0]:(sortByValue)?sortByValue:null;
        if (sortBy) {
            this.defaultSortColumn = sortBy;
            UtilService.localStore({ type: 'set', key: this.lsKeySortColumn ,value:sortBy});
            let columnDetails = _.find(this.state.columnMap, {displayName:sortBy.id});
            if(columnDetails) {
                this.orderBy = 'ordering='+((sortBy.desc)?'-':'')+columnDetails.orgField;
            }
        }
        this.filterQry = this.filterQry.substring(0,this.filterQry.length-1);
       
        this.currentPageSize = (state && state.pageSize) ? state.pageSize : this.currentPageSize;
        let offset = (state && state.pageIndex) ? state.pageIndex*this.currentPageSize : 0;
        await this.getReservationList(this.filterQry, this.orderBy, this.currentPageSize, offset);
        return [this.state.reservationsList, this.totalPage];
    }

    /**
     * Get reservation list
     */
    async getReservationList(filterQry, orderBy, limit, offset) {
        const promises = [  
            ReservationService.getReservationsWithFilter(filterQry, orderBy, limit, offset),
        ];
        const permission = await AuthUtil.getUserRolePermission();
        this.setState({userrole: permission});
             
        this.reservations = [];
        Promise.all(promises).then(async responses => {
            this.totalPage = responses[0].data?responses[0].data.count:0;
            for( const response  of responses[0].data.results){
                let reservation = this.updateReservation(response, response);
                this.reservations.push(reservation);
            };
            this.cycleList.map(cycle => {
                cycle['url'] = cycle.name;
            });
            this.pageUpdated = true;
            this.setState({
                isLoading: false,
                reservationsList: this.reservations,
                cycleList: this.cycleList
            });
        });
    }

    updateReservation (reservation, response) {
        reservation = this.mergeResourceWithReservation( reservation, response.specifications_doc.activity) ;
        reservation = this.mergeResourceWithReservation( reservation, response.specifications_doc.effects );
        reservation = this.mergeResourceWithReservation( reservation, response.specifications_doc.schedulability );
        reservation['reservation_type'] = reservation.type;
        if (response.specifications_doc.resources.stations ) {
            reservation['stations'] = response.specifications_doc.resources.stations.join(', ');
        } else {
            reservation['stations'] = '';
        }
        if(reservation.duration === null || reservation.duration === ''){
            reservation.duration = 'Unknown';
            reservation['stop_time']= 'Unknown';
        } else {
            reservation.duration = UnitService.getSecsToDDHHmmss(reservation.duration);
            reservation['stop_time']= moment(reservation['stop_time']).format(UIConstants.CALENDAR_DATETIME_FORMAT);
        }
        reservation['start_time']= moment(reservation['start_time']).format(UIConstants.CALENDAR_DATETIME_FORMAT);
        reservation['actionpath'] = `/reservation/view/${reservation.id}`;
        reservation['canSelect'] = true;
        return reservation;
    }

    /**
     * Get Option-list values for Select Dropdown filter in 'Viewtable'
     * @param {String} id : Column id
     * @returns 
     */
    getFilterOptions(id) {
        let options = null;
        if(id && id === 'Reservation Type') {
            options = this.reservationTypeOptionList;
        }   else if (id && id === 'Subject') {
            options = this.subjectsOptionList;
        }   else if (id === 'Stations') {
            this.stationList = _.orderBy(this.stationList, ['name'], ['asc']);
            return this.stationList;
        }
        return options;
    }


    /**
     * Function to call wnen websocket is connected
     */
     onConnect() {
        try{
            console.log("WS Opened");
            const userDets = localStorage.getItem("user");
            if (userDets) {
                this.websocket.sendMessage(JSON.stringify({"token": JSON.parse(userDets).websocket_token}));
                console.log("Auth token submitted");
            }
        }catch (err) {
            console.log('err',err)
        }
    }

    /**
     * Function to call when websocket is disconnected
     */
    onDisconnect() {
        console.log("WS Closed")
    }

    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    handleData(data) {
        if (data) {
            console.log('received websocket data:', data)
            const jsonData = JSON.parse(data);
            this.refreshReservationList(jsonData); 
        }
    }

    /**
     * Refresh the Reservation list
     * @param {*} jsonData 
     */
    async refreshReservationList(jsonData) {
        switch (jsonData.object_type) {
            case 'reservation': {
                let reservationList = this.state.reservationsList;
                if (jsonData.action === 'delete') {
                    const count = reservationList.length;
                    _.remove(reservationList, function (reservation) { return reservation.id === jsonData.object_details.id });
                    if (count > reservationList.length) {
                        this.totalPage--;
                    }
                }   else {
                    ReservationService.getReservationsWithFilter(('id='+jsonData.object_details.id+'&'+this.filterQry), this.orderBy, this.currentPageSize, this.offset).then(async (updatedReservation) => {
                        if (updatedReservation && updatedReservation.data.count>0) {
                            let tmpRes = updatedReservation.data.results[0];
                            tmpRes = await this.updateReservation(tmpRes, _.cloneDeep(tmpRes));
                            if (tmpRes) {
                                if (jsonData.action === 'create') {
                                    reservationList.push(tmpRes);
                                    this.totalPage++;
                                }   else if (jsonData.action === 'update') {
                                    _.remove(reservationList, function (reservation) { return reservation.id === tmpRes.id });
                                    reservationList.push(tmpRes);
                                }
                            }
                        }   else {
                            const count = reservationList.length;
                            _.remove(reservationList, function (reservation) { return reservation.id === jsonData.object_details.id });
                            if (count > reservationList.length) {
                                this.totalPage--;
                            }
                        }
                        this.setState({reservationsList: reservationList});
                    });
                }
                
                break;
            }             
            default: { break; }
        }
    }

    render() {
        const permissions = this.state.userrole.userRolePermission.reservation;
        return ( 
            <React.Fragment>
                <PageHeader location={this.props.location} title={'Reservation - List'} 
                           actions={[{icon: 'fa-plus-square', 
                                    title:permissions.create?'Add Reservation': "Don't have permission to add new Reservation",
                                    disabled: permissions.create? !permissions.create: true,
                                    props : { pathname: `/reservation/create`}},
                                     {icon: 'fa-window-close', title:'Click to close Reservation list', type: 'button',  actOn: 'click', props:{ callback: this.closeList }}]}/>     
                 {this.state.isLoading? <AppLoader /> : (this.state.reservationsList) ?
                 <>
                 {permissions.list?
                 <>
                    <div className="p-select " style={{position: 'relative', marginTop: '-2em'}}>
                        <div className="p-field p-grid">
                            <div className="col-lg-3 col-md-3 col-sm-12 ms-height">
                                <span className="p-float-label">
                                    <Dropdown data-testid="cycle" id="cycle" optionLabel="name" optionValue="url"
                                         value={this.state.cycle}  
                                         options={this.state.cycleList} 
                                         onChange={(e) => {this.filterTableData(e.value)}} 
                                         className="ms-width"
                                         filter={true}
                                         showClear={true}
                                         showFilterClear={true}
                                         tooltip="Select cycle to view reservations that start and end in the cycle (will not include those are reserved for unknown duration)"
                                    />
                                    <label htmlFor="cycle" >Filter by Cycle</label>
                                </span>
                            </div>
                            <div className="col-lg-3 col-md-3 col-sm-6 ms-height" style={{ marginLeft: '1em'}}>
                                <span className="p-float-label">
                                    <Flatpickr data-enable-time data-input 
                                        id="fstartdate"
                                        style={{width: '12em'}}
                                        options={{  "inlineHideInput": true,
                                                    "wrap": true,
                                                    "enableSeconds": true,
                                                    "time_24hr": true,
                                                    "minuteIncrement": 1,
                                                    "allowInput": true,
                                                    "defaultHour": 0,
                                                    "plugins": [new confirmDatePlugin()]
                                                }}
                                        title="Select Reserved Between - From"
                                        value={this.state.fStartTime}
                                        onClose={this.setDateRange}
                                        onChange={date => {this.setState({fStartTime :date})}}
                                    >
                                        <input type="text" data-input className={`p-inputtext p-component calendar-input`} placeholder="Reserved - From" />
                                        <button className="p-button p-component p-button-icon-only calendar-button" data-toggle
                                                title="Click to select the date range" >
                                                <i className="fas fa-calendar"></i>
                                        </button>
                                        <button className="p-button p-component p-button-icon-only calendar-reset" 
                                            onClick={this.resetStartDateTime} 
                                            title="Clear date range" >
                                            <i className="fa fa-times" style={{color:'white', marginTop:'-2.85px'}} ></i>
                                        </button>
                                    </Flatpickr> 
                                </span> 
                                <label className={this.state.errors.fStartTime?"error":"info"} style={{position: 'relative', bottom: '27px'}}>
                                    {this.state.errors.fStartTime ? this.state.errors.fStartTime : ""}
                                </label>
                            </div>
                            <div className="col-lg-3 col-md-3 col-sm-6 ms-height" style={{ marginLeft: '2em'}}>
                                <span className="p-float-label">
                                    <Flatpickr data-enable-time data-input 
                                            id="fenddate"
                                            style={{width: '12em'}}
                                            options={{  "inlineHideInput": true,
                                                        "wrap": true,
                                                        "enableSeconds": true,
                                                        "time_24hr": true,
                                                        "minuteIncrement": 1,
                                                        "allowInput": true,
                                                        "defaultHour": 0,
                                                        "plugins": [new confirmDatePlugin()]
                                                    }}
                                            title="Select Reserved Between-To"
                                            value={this.state.fEndTime}
                                            onClose={this.setDateRange}
                                            onChange={date => {this.setState({fEndTime :date})}}
                                        >
                                            <input type="text" data-input className={`p-inputtext p-component calendar-input`} placeholder="Reserved - To" />
                                            <button className="p-button p-component p-button-icon-only calendar-button" data-toggle
                                                    title="Click to select the date range" >
                                                    <i className="fas fa-calendar"></i>
                                            </button>
                                            <button className="p-button p-component p-button-icon-only calendar-reset" 
                                                onClick={this.resetEndDateTime} title="Clear date range" >
                                                <i className="fa fa-times" style={{color:'white', marginTop:'-2.85px'}} ></i>
                                            </button>
                                        </Flatpickr>
                                </span>
                                <label className={this.state.errors.fEndTime?"error":"info"} style={{position: 'relative', bottom: '27px'}} >
                                    {this.state.errors.fEndTime ? this.state.errors.fEndTime : ""}
                                </label>
                            </div>
                        </div>

                    </div>
                    <div className="delete-option">
                        <div >
                            <span className="p-float-label">
                                <a href="#" onClick={permissions.delete?this.confirmDeleteReservations:()=>{}}
                                    title={permissions.delete?"Delete selected Reservation(s)":"Don't have delete permission"}>
                                    <i className={`fa fa-trash${permissions.delete?'':' fa-disabled'}`} aria-hidden="true" ></i>
                                </a>
                            </span>
                        </div>                           
                    </div>
                    <ViewTable 
                        data={this.state.reservationsList} 
                        defaultcolumns={this.state.tmpDefaulcolumns ? this.state.tmpDefaulcolumns : this.state.defaultcolumns} 
                        optionalcolumns={this.state.tmpOptionalcolumns ? this.state.tmpOptionalcolumns : this.state.optionalcolumns}
                        columnclassname={this.state.columnclassname}
                        columnOrders={this.state.columnOrders}
                        defaultSortColumn={this.defaultSortColumn}
                        showaction="true"
                        paths={this.state.paths}
                        tablename="reservation_list"
                        showCSV= {true}
                        allowRowSelection={true}
                        onRowSelection = {this.onRowSelection}
                        toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                        lsKeySortColumn={this.lsKeySortColumn}
                        pageUpdated={this.pageUpdated}
                        storeFilter={true}
                        callBackFunction={this.fetchTableData}
                        showFilterOption={this.getFilterOptions}
                        totalPage={this.totalPage}
                        clearAllFuncCallback={this.clearAllFilter} // Callback function will call when the clearAllFilter called in ViewTable
                        tableInstanceRef={this.seTableInstanceRef}
                    />
                    </>: <AccessDenied/>}
                </>
                : <div>No Reservation found </div>
                }

                <CustomDialog type="confirmation" visible={this.state.dialogVisible}
                    header={this.state.dialog.header} message={this.state.dialog.detail} actions={this.state.dialog.actions}
                    content={this.state.dialog.content} width={this.state.dialog.width} showIcon={this.state.dialog.showIcon}
                    onClose={this.closeDialog} onCancel={this.closeDialog} onSubmit={this.state.dialog.onSubmit}/>
                {!this.state.isLoading &&
                    <Websocket ref={websocket => this.websocket = websocket} url={process.env.REACT_APP_WEBSOCKET_URL} 
                        onOpen={this.onConnect} onMessage={this.handleData} onClose={this.onDisconnect} />}
            </React.Fragment>
        );
    }
}
