import {getDailyColumns} from "./excelview.schedulingset.helper";

describe('getDailyColumns', () => {
    test.each([null, undefined])('Returns empty array when dailyConstraint is: %s', (dailyConstraint) => {
        const result = getDailyColumns(dailyConstraint);
        expect(result).toEqual([]);
    })

    test('should return an empty array when no properties in dailyConstraint are set to true', () => {
        const dailyConstraint = {
            require_day: false,
            require_night: false,
            avoid_twilight: false
        };
        const result = getDailyColumns(dailyConstraint);
        expect(result).toEqual([]);
    });

    test('should return ["require_day"] when dailyConstraint.require_day is true', () => {
        const dailyConstraint = {
            require_day: true,
            require_night: false,
            avoid_twilight: false
        };
        const result = getDailyColumns(dailyConstraint);
        expect(result).toEqual(['require_day']);
    });

    test('should return an array with all column names when all properties in dailyConstraint are set to true', () => {
        const dailyConstraint = {
            require_day: true,
            require_night: true,
            avoid_twilight: true
        };
        const result = getDailyColumns(dailyConstraint);
        expect(result).toEqual(['require_day', 'require_night', 'avoid_twilight']);
    });

    test('should return an empty array when the properties in dailyConstraint are unknown', () => {
        const dailyConstraint = {
            schaap: true,
            requireDay: true,
        };
        const result = getDailyColumns(dailyConstraint);
        expect(result).toEqual([]);
    });
});
