import React, { Component } from 'react';
import { Redirect } from 'react-router-dom'

import { Button } from 'primereact/button';
import { Dropdown } from 'primereact/dropdown';
import { InputText } from 'primereact/inputtext';
import { InputTextarea } from 'primereact/inputtextarea';
import moment from 'moment';
import _ from 'lodash';
import Flatpickr from "react-flatpickr";
import { Link } from 'react-router-dom';
import { publish } from '../../App';
import { CustomDialog } from '../../layout/components/CustomDialog';
import { appGrowl } from '../../layout/components/AppGrowl';
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import Jeditor from '../../components/JSONEditor/JEditor';
import UIConstants from '../../utils/ui.constants';
import TaskService from '../../services/task.service';
import SystemEventService from '../../services/system.event.service';
import UtilService from '../../services/util.service';
import { InputMask } from 'primereact/inputmask';
import UnitConverter from '../../utils/unit.converter';
import {AffectedTasksSelector} from './affeted.tasks.selector';
import { Chips } from 'primereact/chips';
import { Dialog } from 'primereact/dialog';
import FormActionbar from '../../components/FormActionbar'; 

export class SystemEventEdit extends Component {
    TASK_BLUEPRINT_EXPAND = 'scheduling_unit_blueprint,subtasks';
    TASK_BLUEPRINT_FIELDS = ['id', 'url','name', 'short_description', 'on_sky_start_time', 'on_sky_stop_time', 'scheduling_unit_blueprint', 'scheduling_unit_blueprint.id', 
                            'scheduling_unit_blueprint.name','subtasks.id','subtasks.primary'];
    constructor(props) {
        super(props);
        this.state = {
            isLoading: true,
            isDirty: false,
            errors: {},                             // Validation Errors
            validFields: {},                        // For Validation
            validForm: false,                       // To enable Save Button
            validEditor: false,
            dialog: { header: '', detail: '' },      // Dialog properties
            affectedTaskChips: [],
            dialogFilters: {}
        };
        this.seAffectedHardwareTemplateList = [];
        this.affectedTaskList = [];
        this.seIssueTypeList = [];
        this.seIssueSubTypeList = [];
        this.seSeverityList = [];
        this.seStatusList = [];
        this.affectedHardwareTemplates = [];
        this.systemeventTemplate = [];

        this.tooltipOptions = UIConstants.tooltipOptions;
        this.setEditorOutput = this.setEditorOutput.bind(this);
        this.setEditorFunction = this.setEditorFunction.bind(this);
        this.checkIsDirty = this.checkIsDirty.bind(this);
        this.updateSystemEvent = this.updateSystemEvent.bind(this);
        this.close = this.close.bind(this);
        this.cancelEdit = this.cancelEdit.bind(this);
        this.isValidDuration = this.isValidDuration.bind(this);
        this.synchFilter = this.synchFilter.bind(this);
        this.showAffectedTasksSelection = this.showAffectedTasksSelection.bind(this);
        this.selectAffectedTasks = this.selectAffectedTasks.bind(this);
        this.addAffectedTasks = this.addAffectedTasks.bind(this);
        this.removeChipsItem = this.removeChipsItem.bind(this);
        this.resetAffectedtasks = this.resetAffectedtasks.bind(this);
        this.tagSorting = this.tagSorting.bind(this);

       // Validateion Rules
        this.formRules = {
            name: { required: true, message: "Name can not be empty" },
            issue_type: {required: true, message: "Issue Type can not be empty"},
            issue_subtype: { required: true, message: "Issue SubtType can not be empty" },
            severity: {required: true, message: "Severity can not be empty"},
            status: {required: true, message: "Status can not be empty"},
            start: { required: true, message: "Start Time can not be empty" },
        };
    }

    componentDidMount() {
        const promises = [
            SystemEventService.getSEAffectedHardwareTemplates(),
            SystemEventService.getSEIssueTypes(),
            SystemEventService.getSEIssueSubTypes(),
            SystemEventService.getSESeverity(),
            SystemEventService.getSEStatus(),
            UtilService.getUTC(),
            ];
        Promise.all(promises).then(responses => {
            this.seAffectedHardwareTemplateList = responses[0];
            this.seIssueTypeList = responses[1];
            this.seIssueSubTypeList = responses[2];
            this.seSeverityList = responses[3];
            this.seStatusList = responses[4];
            let systemTime = moment.utc(responses[5]);
            this.setState({
                systemTime: systemTime
            });
        });
        // Get system event details
        this.getSystemEventDetails();
    }

    /**
     * JEditor's function that to be called when parent wants to trigger change in the JSON Editor
     * @param {Function} editorFunction 
     */
    setEditorFunction(editorFunction) {
        this.setState({ editorFunction: editorFunction });
    }
  
    /**
     * To get the system event details from the backend using the service
     * 
     */
    getSystemEventDetails() {
        let affectedTaskChips = [];
        let existingAffectedTaskList = [];
        const systemeventId = this.props.match ? this.props.match.params.id : null;
        if (systemeventId) {
            SystemEventService.getSystemEvent(systemeventId)
            .then(async (systemevent) => {
                if (systemevent) {
                    if (systemevent.affected_tasks_ids && systemevent.affected_tasks_ids.length>0) {
                        const response = await TaskService.getExpandedTasksWithFilter('blueprint', this.TASK_BLUEPRINT_EXPAND, this.TASK_BLUEPRINT_FIELDS, "id="+(systemevent.affected_tasks_ids.toString()), '', systemevent.affected_tasks_ids.length, 0);
                        if (response.data.results && response.data.results.length>0) {
                            existingAffectedTaskList = response.data.results;
                            response.data.results.map(row => {
                                const controlTask = _.find(row.subtasks, subtask => {return subtask.primary===true});
                                const displayName = `${row.name} [${row.id}] - ${row.short_description} [${controlTask?controlTask.id:''}]`;
                                affectedTaskChips.push(<Link target="_blank" id={row.id} title={"View the Task "+row.name+' ('+row.id+')'} 
                                to={{ pathname: `/task/view/blueprint/${row.id}` }} style={{color: 'white', whiteSpace: 'pre-wrap'}}>{displayName}</Link>)
                            });
                        }
                    }
                    SystemEventService.getSystemEventTemplate(systemevent.affected_hardware_template_id)
                    .then((systemeventTemplate) => {
                        systemevent['tasksId'] = systemevent.affected_tasks_ids.length>0?systemevent.affected_tasks_ids.join():'';
                        systemevent['duration'] = systemevent.stop?UnitConverter.getDuration(systemevent.start, systemevent.stop):'Unknown';
                        affectedTaskChips.sort(this.tagSorting());
                        this.setState({
                            redirect: null, 
                            systemevent: systemevent, 
                            isLoading: false, 
                            systemeventTemplate: systemeventTemplate,
                            paramsSchema: systemeventTemplate.ref_resolved_schema,
                            affectedTaskChips:affectedTaskChips,
                            existingAffectedTaskList: existingAffectedTaskList
                        });
                    });
                }   else {
                    this.setState({redirect: "/not-found"});
                }
            });
        }   else {
            this.setState({redirect: "/not-found"});
        }
    }

    close() {
        this.setState({ showDialog: false });
    }

    /**
     * Cancel edit and redirect to system event View page
     */
    cancelEdit() {      
        publish('edit-dirty', false);
        this.props.history.length>1?this.state.isDirty?this.props.history.go(-2):this.props.history.go(-1):this.props.history.push(`/systemevent/view/${this.props.match.params.id}`);
        this.setState({showDialog: false});
    }

    /**
     * warn before cancel this page if any changes detected 
     */
    checkIsDirty() {
        if (this.state.isDirty) {
            this.setState({ showDialog: true });
        } else {
            this.cancelEdit();
        }
    }

    /**
     * Validation function to validate the form or field based on the form rules.
     * If no argument passed for fieldName, validates all fields in the form.
     * @param {string} fieldName 
     */
    validateForm(fieldName) {
        let validForm = false;
        let errors = this.state.errors;
        let validFields = this.state.validFields;
        if (fieldName) {
            delete errors[fieldName];
            delete validFields[fieldName];
            if (this.formRules[fieldName]) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.systemevent[fieldName];
                if (rule.required) {
                    if (!fieldValue) {
                        errors[fieldName] = rule.message ? rule.message : `${fieldName} is required`;
                    } else {
                        validFields[fieldName] = true;
                    }
                }
            }
        } else {
            errors = {};
            validFields = {};
            for (const fieldName in this.formRules) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.systemevent[fieldName];
                if (rule.required) {
                    if (!fieldValue) {
                        errors[fieldName] = rule.message ? rule.message : `${fieldName} is required`;
                    } else {
                        validFields[fieldName] = true;
                    }
                }
            }
        }
        this.setState({ errors: errors, validFields: validFields });
        if (Object.keys(validFields).length === Object.keys(this.formRules).length) {
            validForm = true;
            delete errors['start'];
            delete errors['stop'];
        }
        if (!this.validateDates(this.state.systemevent.start, this.state.systemevent.stop)) {
            validForm = false;
            if (!fieldName || fieldName === 'start') {
                errors['start'] = "Start Time cannot be same or after End Time";
                delete errors['stop'];
            }
            if (!fieldName || fieldName === 'stop') {
                errors['stop'] = "End Time cannot be same or before Start Time";
                delete errors['start'];
            }
            this.setState({ errors: errors });
        }
        if (fieldName === 'duration' && this.state.systemevent.duration) {
            var values = this.state.systemevent.duration.split(' ');
            var days = values[0];
            var dValues = values[1].split(':');           
            delete errors['duration'];
            if ((days *1 )===0 && (dValues[0] * 1) === 0 && (dValues[1] * 1) === 0 && (dValues[2] * 1) === 0) {
                validForm = false;
                if (!fieldName || fieldName === 'duration') {
                    errors['duration'] = "Duration cannot be zero";
                }
                this.setState({ errors: errors });
            }
        }
        return validForm;
    }

    /**
     * Function to validate if stop is always later than start if exists.
     * @param {Date} start 
     * @param {Date} stop 
     * @returns boolean
     */
    validateDates(start, stop) {
        if (start && stop && moment(stop).isSameOrBefore(moment(start))) {
            return false;
        }
        return true;
    }

    /**
    * This function is mainly added for Unit Tests. If this function is removed Unit Tests will fail.
    */
    validateEditor() {
        return this.validEditor;
    }

    /**
     * Function to call on change and blur events from input components
     * @param {string} key 
     * @param {any} value 
     */
    setParams(key, value, type) {
        let systemevent = this.state.systemevent;
        switch (type) {
            case 'NUMBER': {
                systemevent[key] = value ? parseInt(value) : 0;
                break;
            }
            default: {
                systemevent[key] = value;
                break;
            }
        }
        if(!this.state.isDirty) {
            publish('edit-dirty', true);
        }
        this.setState({ systemevent: systemevent, validForm: this.validateForm(key), isDirty: true },
        () => {
            this.setDurationOrEndValue(key);
        });
    }

    /**
     * Function to set the value for the dependant fields when value is set to one field.
     * When start or stop is changed, duration will be updated accordingly.
     * Similarly if duration is changed, stop is updated.
     * @param {String} key - property name of the systemevent
     */
    setDurationOrEndValue = (key) => {
        if ( key === 'start' || key === 'stop') {
            if (this.state.systemevent.start && this.state.systemevent.stop) {
                var delta = Math.abs(new Date(this.state.systemevent.start) - new Date(this.state.systemevent.stop)) / 1000;
                let tempDuration = UnitConverter.getSecsToDDHHmmss(delta);              
                this.setDurationOrStopTime('duration', tempDuration);
            }   else if (key === 'start' && this.state.systemevent.start && this.state.systemevent.duration) {
                let stopDate = UnitConverter.getEndDateFromDuration(this.state.systemevent.start, this.state.systemevent.duration);               
                this.setDurationOrStopTime('stop', stopDate);
            }   else if (key === 'stop' && !this.state.systemevent.stop) {
                this.setDurationOrStopTime('duration', "");
            }
        }
        else if (key === 'duration') {            
            if (this.state.systemevent.start) {
                let stopDate = UnitConverter.getEndDateFromDuration(this.state.systemevent.start, this.state.systemevent.duration);               
                this.setDurationOrStopTime('stop', stopDate);
            }
        }
    }
    
    /**
     * Function to set calcualted value for either duration or stop_time
     * @param {String} key - name of the field
     * @param {*} value - value to set for the field
     */
    setDurationOrStopTime(key, value) {
        let systemevent = this.state.systemevent;
        systemevent[key] = value;
        this.setState({ systemevent: systemevent, validForm: this.validateForm(key), isDirty: true });
    }

    /**
     * Function to validate the duration field.
     * @param {String} value - Duration in format 'Days HH:mm:ss'
     * @returns boolean
     */
    isValidDuration(value) {
        let errors = this.state.errors;
        let touched = this.state.touched;
        let systemevent = this.state.systemevent;
        let validForm = this.state.validForm;
        if (value.length === 12 && (value === "000 00:00:00" ||
            !value.match(/^([0-1]90 00:00:00)|([0-1][0-8][0-9] ([0-1]?\d|2[0-3]):([0-5]?\d):([0-5]?\d))$/))) {
            errors.duration = "Not valid duration. Duration should be in Days Hours:minutes:seconds. Min - 000 00:00:01, Max - 190 00:00:00";
            touched.duration = true;
            validForm = false;
        }   else {
            delete errors["duration"];
            delete touched["duration"];
            validForm = this.validateForm();
        }
        systemevent.duration = value;
        this.setState({errors: errors, touched: touched, systemevent: systemevent, validForm: validForm});
        return errors.duration?false:true;
    }

    /**
     * Set JEditor output
     * @param {*} jsonOutput 
     * @param {*} errors 
     */
    setEditorOutput(jsonOutput, errors) {
        this.paramsOutput = jsonOutput;
        this.validEditor = errors.length === 0;
        if (!this.state.isDirty && this.state.paramsOutput && !_.isEqual(this.state.paramsOutput, jsonOutput)) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({
                paramsOutput: jsonOutput,
                validEditor: errors.length === 0,
                validForm: this.validateForm(),
                isDirty: true
            });
        } else {
            this.setState({
                paramsOutput: jsonOutput,
                validEditor: errors.length === 0,
                validForm: this.validateForm()
            });
        }
    }

     /**
     * Function to set form values to the system svent object
     * @param {string} key 
     * @param {object} value 
     */
    setSystemEventParams(key, value) {
        let systemevent = _.cloneDeep(this.state.systemevent);
        systemevent[key] = value;
        if (!this.state.isDirty && !_.isEqual(this.state.systemevent, systemevent)) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({
                systemevent: systemevent, validForm: this.validateForm(key), validEditor: this.validateEditor(), touched: {
                    ...this.state.touched,
                    [key]: true
                }, isDirty: true
            });
        } else {
            this.setState({
                systemevent: systemevent, validForm: this.validateForm(key), validEditor: this.validateEditor(), touched: {
                    ...this.state.touched,
                    [key]: true
                }
            });
        }
    }    

    /**
     * Update system event
     */
    async updateSystemEvent() {
        let systemevent = this.state.systemevent;
        systemevent['start'] = moment(systemevent['start']).format(UIConstants.CALENDAR_DATETIME_FORMAT);
        systemevent['stop'] = (systemevent['stop'] && systemevent['stop'] !== 'Invalid date') ? moment(systemevent['stop']).format(UIConstants.CALENDAR_DATETIME_FORMAT) : null;
        systemevent['affected_hardware_doc'] = this.paramsOutput;
        let affected_tasks = [];
        if (this.state.affectedTaskChips) {
            for (const selectedTask of this.state.affectedTaskChips) {
                const leftIndex = selectedTask.props.children.indexOf("[");
                const rightIndex = selectedTask.props.children.indexOf("]");
                let id = selectedTask.props.children.substring(leftIndex+1,rightIndex);
                affected_tasks.push(`/api/task_blueprint/${id}`);
            }
        }
        systemevent['affected_tasks'] = affected_tasks;
        systemevent['jira_url'] = systemevent.jira_url && systemevent.jira_url.length >0 ? systemevent.jira_url : null;
        systemevent['notes'] = systemevent.notes && systemevent.notes.length >0 ? systemevent.notes : null;
        systemevent = await SystemEventService.updateSystemEvent(systemevent);
        if (systemevent && systemevent.id) {
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'System Event updated successfully.' });
            this.props.history.push({
                pathname: `/systemevent/view/${this.props.match.params.id}`,
            });
            publish('edit-dirty', false);
        } else {
            appGrowl.show({ severity: 'error', summary: 'Error Occured', detail: 'Unable to update System Event', showDialog: false, isDirty: false });
        }
    }

    /**
     * To show 'Select Affected Tasks' dialog
     */
    showAffectedTasksSelection() {
        let dialog = this.state.dialog;
        dialog.header = "Select Affected Tasks";
        dialog.detail = <AffectedTasksSelector startTime={this.state.systemevent.start} stopTime={this.state.systemevent.stop} onRowSelection={this.selectAffectedTasks} 
        dialogFilters={this.state.dialogFilters} synchFilter={this.synchFilter}/>
        dialog.style = {width: '70vw', height: '50vw'};
        dialog.cancelLabel = 'Cancel';
        dialog.submitLabel = 'Add Task(s)';
        dialog.onsubmit = this.addAffectedTasks;
        dialog.onCancel = null;
        dialog.showAffectedTasks = true;
        this.setState({
            dialog: dialog,
            dialogVisible: true
        });
    }

    /**
     * Assign the selected rows in UI table
     * @param {json} selectedTasks 
     */
    selectAffectedTasks(selectedTasks) {
        this.affectedTaskList = selectedTasks;
    }
    
    /**
     * Prepare Chips for UI using selected rows
     */
    addAffectedTasks() {
        let affectedTaskChips = [];
        if (this.affectedTaskList && this.affectedTaskList.length>0){
            affectedTaskChips = _.cloneDeep(this.state.affectedTaskChips);
            affectedTaskChips = affectedTaskChips?affectedTaskChips:[];
            this.affectedTaskList.map(row => {
                const existingTask = _.find(this.state.affectedTaskChips, (task) => {return task.props.id === row.id});
                if(existingTask === undefined || existingTask.length === 0) {
                    const displayName = `${row.name} [${row.id}] - ${row.short_description} [${row.subtasks_0_id}]`;
                    affectedTaskChips.push(<Link target="_blank" id={row.id} title={"View the Task "+row.name+' ('+row.id+')'}
                    to={{ pathname: `/task/view/blueprint/${row.id}` }} style={{color: 'white', whiteSpace: 'pre-wrap'}}>{displayName}</Link>);
                }
            });
            affectedTaskChips.sort(this.tagSorting());
            this.setState({affectedTaskChips: [...new Set(affectedTaskChips)], dialogVisible: false, isDirty: true});
        }   else {
            appGrowl.show({severity: 'info', summary: 'Info', detail: 'Select at least one task and click apply'});
        }
        this.affectedTaskList = [];       
    }

    /**
     * Task sorting by id
     * @returns 
     */
    tagSorting() {    
        return function(nextTask, previoustask) {    
            if (nextTask.props.id > previoustask.props.id) {    
                return 1;    
            } else if (nextTask.props.id < previoustask.props.id) {    
                return -1;    
            }    
            return 0;    
        }    
    }

    /**
     * Remove Chips form UI
     * @param {*} value 
     */
    removeChipsItem(value) {
        let affectedTaskChips = _.remove( _.cloneDeep(this.state.affectedTaskChips), function(task) {return  (task.props.children !== value[0].props.children);});
        this.setState({affectedTaskChips: affectedTaskChips, isDirty: true});
        this.affectedTaskList = [];
    }

    /**
     * Reset with original value
     */
    resetAffectedtasks() {
        this.affectedTaskList = [];
        let affectedTaskChips = [];
        this.setState({affectedTaskChips: affectedTaskChips, isDirty: true});
    }

    /**
     * Synch the filter used in dialog, this will be reset to default when user navigate to another page
     */
    synchFilter(filters) {
        this.setState({dialogFilter: filters});
    }

    render() {
        if (this.state.redirect) {
            return <Redirect to={{ pathname: this.state.redirect }}></Redirect>
        }
        let jeditor = null;
        if (this.state.systemeventTemplate) {
            if (this.state.systemevent.affected_hardware_doc.$id) {
                delete this.state.systemevent.affected_hardware_doc.$id;
                delete this.state.systemevent.affected_hardware_doc.$schema;
            }
            jeditor = React.createElement(Jeditor, {
                title: "System Event Parameters",
                schema: this.state.systemeventTemplate.ref_resolved_schema,
                initValue: this.state.systemevent.affected_hardware_doc,
                disabled: false,
                callback: this.setEditorOutput,
                parentFunction: this.setEditorFunction
            });
        }

        return (
            <React.Fragment>
                <PageHeader location={this.props.location} title={'System Event - Edit'} actions={[{
                    icon: 'fa-window-close',
                    title: 'Click to Close System Event - Edit', type: 'button', actOn: 'click', props: { callback: this.checkIsDirty }
                }]} />

                { this.state.isLoading ? <AppLoader /> : this.state.systemevent &&
                    <React.Fragment>
                        <div>
                            <div className="p-fluid" style={{marginBottom: "1em"}}>
                                <div className="p-field p-grid">
                                    <label htmlFor="systemeventname" className="col-lg-2 col-md-2 col-sm-12">Name <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <InputText className={(this.state.errors.name && this.state.touched.name) ? 'input-error' : ''} id="systemeventname" data-testid="name"
                                            tooltip="Enter name of the System Event" tooltipOptions={this.tooltipOptions} maxLength="128"
                                            ref={input => { this.nameInput = input; }}
                                            value={this.state.systemevent.name} autoFocus
                                            onChange={(e) => this.setSystemEventParams('name', e.target.value)}
                                            onBlur={(e) => this.setSystemEventParams('name', e.target.value)}
                                        />
                                        <label className={(this.state.errors.name && this.state.touched.name) ? "error" : "info"}>
                                            {this.state.errors.name && this.state.touched.name ? this.state.errors.name : "Max 128 characters"}
                                        </label>
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label htmlFor="description" className="col-lg-2 col-md-2 col-sm-12">Description <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <InputTextarea className={(this.state.errors.description && this.state.touched.description) ? 'input-error' : ''} rows={3} cols={30}
                                            tooltip="Longer description of the System Event"
                                            tooltipOptions={this.tooltipOptions}
                                            maxLength="128"
                                            data-testid="description"
                                            value={this.state.systemevent.description}
                                            onChange={(e) => this.setSystemEventParams('description', e.target.value)}
                                            onBlur={(e) => this.setSystemEventParams('description', e.target.value)}
                                        />
                                        <label className={(this.state.errors.description && this.state.touched.description) ? "error" : "info"}>
                                            {(this.state.errors.description && this.state.touched.description) ? this.state.errors.description : "Max 255 characters"}
                                        </label>
                                    </div>
                                </div>

                                <div className="p-field p-grid">
                                    <label htmlFor="projCat" className="col-lg-2 col-md-2 col-sm-12">Issue Type <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12 systemEvent" data-testid="projCat" >
                                        <Dropdown inputId="projCat" optionLabel="value" optionValue="url" 
                                                tooltip="Issue Type" tooltipOptions={this.tooltipOptions}
                                                value={this.state.systemevent.issue_type} 
                                                options={this.seIssueTypeList} 
                                                onChange={(e) => {this.setSystemEventParams('issue_type', e.value)}} 
                                                placeholder="Select Issue Type" />
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label htmlFor="periodCategory" className="col-lg-2 col-md-2 col-sm-12">Issue Subtype  <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12 systemEvent">
                                        <Dropdown data-testid="period-cat" id="period-cat" optionLabel="value" optionValue="url" 
                                                tooltip="Issue Subtype" tooltipOptions={this.tooltipOptions}
                                                value={this.state.systemevent.issue_subtype}
                                                options={this.seIssueSubTypeList} 
                                                onChange={(e) => {this.setSystemEventParams('issue_subtype',e.value)}} 
                                                placeholder="Select Issue Subtype" />
                                    </div>
                                </div> 
                                <div className="p-field p-grid">
                                    <label htmlFor="systemEventName" className="col-lg-2 col-md-2 col-sm-12">Jira URL </label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <InputText className={(this.state.errors.jira_url && this.state.touched.jira_url) ? 'input-error' : ''} id="systemEventName" data-testid="name"
                                            tooltip="Enter Jira URL" tooltipOptions={this.tooltipOptions} maxLength="255"
                                            ref={input => { this.nameInput = input; }}
                                            value={this.state.systemevent.jira_url}
                                            onChange={(e) => this.setSystemEventParams('jira_url', e.target.value)}
                                            onBlur={(e) => this.setSystemEventParams('jira_url', e.target.value)} 
                                            />
                                        <label className={(this.state.errors.jira_url && this.state.touched.jira_url) ? "error" : "info"}>
                                            {this.state.errors.jira_url && this.state.touched.jira_url ? this.state.errors.jira_url : "Max 255 characters"}
                                        </label>
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label htmlFor="notes" className="col-lg-2 col-md-2 col-sm-12">Notes </label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <InputTextarea className={(this.state.errors.notes && this.state.touched.notes) ? 'input-error' : ''} rows={3} cols={30}
                                                tooltip="Notes of the System Event"
                                                tooltipOptions={this.tooltipOptions}
                                                maxLength="255"
                                                data-testid="notes"
                                                value={this.state.systemevent.notes}
                                                onChange={(e) => this.setSystemEventParams('notes', e.target.value)}
                                                onBlur={(e) => this.setSystemEventParams('notes', e.target.value)} 
                                        />
                                        <label className={(this.state.errors.notes && this.state.touched.notes) ? "error" : "info"}>
                                            {(this.state.errors.notes && this.state.touched.notes) ? this.state.errors.notes : "Max 255 characters"}
                                        </label>
                                    </div>
                                </div>
                                <div className="p-field p-grid">
                                    <label htmlFor="severity" className="col-lg-2 col-md-2 col-sm-12">Severity  <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12 systemEvent">
                                        <Dropdown data-testid="severity" id="severity" optionLabel="value" optionValue="url" 
                                                tooltip="Status" tooltipOptions={this.tooltipOptions}
                                                value={this.state.systemevent.severity} 
                                                options={this.seSeverityList} 
                                                onChange={(e) => {this.setSystemEventParams('severity',e.value)}} 
                                                placeholder="Select System Event Severity" />
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label htmlFor="status" className="col-lg-2 col-md-2 col-sm-12">Status <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12 systemEvent">
                                        <Dropdown data-testid="status" id="status" optionLabel="value" optionValue="url" 
                                                tooltip="Status" tooltipOptions={this.tooltipOptions}
                                                value={this.state.systemevent.status} 
                                                options={this.seStatusList} 
                                                onChange={(e) => {this.setSystemEventParams('status',e.value)}} 
                                                placeholder="Select System Event Status" />
                                    </div>                                    
                                </div>
                                <div className="p-field p-grid">
                                    <label className="col-lg-2 col-md-2 col-sm-12">Start Time <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <Flatpickr data-enable-time data-input 
                                            options={{
                                                "inlineHideInput": true,
                                                "wrap": true,
                                                "enableSeconds": true,
                                                "time_24hr": true,
                                                "minuteIncrement": 1,
                                                "allowInput": true,
                                            }}
                                            title="Start of this system event"
                                            value={this.state.systemevent.start}
                                            onChange={value => {
                                                this.setParams('start', value[0] ? value[0] : this.state.systemevent.start);
                                                this.setSystemEventParams('start', value[0] ? value[0] : this.state.systemevent.start)
                                            }} >
                                            <input type="text" data-input className={`p-inputtext p-component ${this.state.errors.start && this.state.touched.start ? 'input-error' : ''}`} />
                                            <i className="fa fa-calendar" data-toggle style={{ position: "absolute", marginLeft: '-25px', marginTop: '5px', cursor: 'pointer' }} ></i>
                                            <i className="fa fa-times" style={{ position: "absolute", marginLeft: '-50px', marginTop: '5px', cursor: 'pointer' }}
                                                onClick={e => { this.setParams('start', ''); this.setSystemEventParams('start', '') }}></i>
                                        </Flatpickr>
                                        {this.state.errors.start && this.state.touched.start &&
                                        <label className={this.state.errors.start && this.state.touched.start ? "error" : "info"}>
                                            {this.state.errors.start && this.state.touched.start ? this.state.errors.start : ""}
                                        </label>}
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label className="col-lg-2 col-md-2 col-sm-12">End Time</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <Flatpickr data-enable-time data-input 
                                            options={{
                                                "inlineHideInput": true,
                                                "wrap": true,
                                                "enableSeconds": true,
                                                "time_24hr": true,
                                                "minuteIncrement": 1,
                                                "allowInput": true,
                                                "minDate": this.state.systemevent.start ? this.state.systemevent.start.toDate : '',
                                            }}
                                            title="End of this system event."
                                            value={this.state.systemevent.stop}
                                            onChange={value => {
                                                this.setParams('stop', value[0] ? value[0] : this.state.systemevent.stop);
                                                this.setSystemEventParams('stop', value[0] ? value[0] : this.state.systemevent.stop)
                                            }} >
                                            <input type="text" data-input className={`p-inputtext p-component ${this.state.errors && this.state.errors.stop && this.state.touched && this.state.touched.stop ? 'input-error' : ''}`} />
                                            <i className="fa fa-calendar" data-toggle style={{ position: "absolute", marginLeft: '-25px', marginTop: '5px', cursor: 'pointer' }} ></i>
                                            <i className="fa fa-times" style={{ position: "absolute", marginLeft: '-50px', marginTop: '5px', cursor: 'pointer' }}
                                                onClick={e => { this.setParams('stop', ''); this.setSystemEventParams('stop', '') }}></i>
                                        </Flatpickr>
                                    </div>
                                </div>
                               
                                <div className="p-field p-grid">
                                    <label className="col-lg-2 col-md-2 col-sm-12">Duration</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <InputMask mask="999 99:99:99"
                                            tooltip="Enter duration in (Days HH:mm:ss) format. Min-000 00:00:01 & Max-190 00:00:00."
                                            tooltipOptions={this.tooltipOptions} 
                                            value={this.state.systemevent.duration}
                                            placeholder="DDD HH:MM:SS"
                                            onChange={(e) =>{if(!e.value) {this.setDurationOrEndValue("stop")}}}
                                            onComplete={(e) => {
                                                if (this.isValidDuration(e.value)) {
                                                    this.setSystemEventParams('duration', e.value);
                                                    this.setParams('duration', e.value ? e.value : this.state.systemevent.duration);
                                                }
                                            }}
                                            onBlur={(e) => {
                                                this.setParams('duration', e.value ? e.value : this.state.systemevent.duration);
                                            }}></InputMask>
                                        <label className={this.state.errors.duration && this.state.touched.duration ? "error" : "info"}>
                                            {this.state.errors.duration && this.state.touched.duration ? this.state.errors.duration : ""}
                                        </label>
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label htmlFor="seat" className="col-lg-2 col-md-2 col-sm-12">Affected Tasks</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <div className="chips">
                                            {this.state.affectedTaskChips && <Chips value={this.state.affectedTaskChips} onRemove={(e) => this.removeChipsItem(e.value)} />}
                                        </div>                                        
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12" style={{display: 'flex', height: '2em'}}>
                                        <Button label="" className="p-button-primary" icon="pi pi-plus" 
                                            tooltip="Add Affected Tasks"
                                            onClick={this.showAffectedTasksSelection} 
                                        />
                                       <Button label="" className="p-button-primary" icon="fas fa-sync-alt" style={{marginLeft: '0.5em'}}
                                                tooltip= {"Remove all Affected Tasks"}
                                                onClick={this.resetAffectedtasks}
                                                disabled={(this.props.schedulingUnit)?false:this.state.affectedTaskChips.length === 0}
                                            />
                                    </div>
                                </div>
                                <div className="p-grid" style={{marginLeft: '0.1em'}}>
                                    <fieldset className="border-style">
                                        <div className="p-col-12" style={{width: "100%"}}>
                                            {this.state.paramsSchema ? jeditor : ""}
                                        </div>
                                    </fieldset>
                                </div>
                            </div>
                            <div className="p-breadcrumb footer-long-div actions" >
                                <FormActionbar onSubmit={this.updateSystemEvent} 
                                enableSaveBtn={!this.state.validEditor || !this.state.validForm} onCancel={this.checkIsDirty} />
                            </div>
                        </div>
                    </React.Fragment>
                }
                <div className="p-grid affectedTask" data-testid="confirm_dialog">
                    <Dialog header={this.state.dialog.header} visible={this.state.dialogVisible} style={this.state.dialog.style} inputId="confirm_dialog"
                        modal={true} onHide={() => { this.setState({ dialogVisible: false }) }}
                        footer={<div>
                                    <Button key="submit" type="primary" onClick={this.state.dialog.onsubmit} label={this.state.dialog.submitLabel} />
                                    <Button key="back" className="act-btn-cancel" onClick={() => {this.state.dialog.onCancel?this.state.dialog.onCancel(): this.setState({ dialogVisible: false }) }} label={this.state.dialog.cancelLabel} />
                                </div>
                                } >
                        {this.state.dialog.showAffectedTasks ?
                            <div className="ag-theme-balham" style={{ height: '75vh', width: '100%'}}>
                                <div className="p-fluid">
                                    <div className="p-grid">
                                        <div className="p-col-12" style={{width: "100%"}}>
                                        {this.state.dialog.detail}
                                        </div>
                                    </div>
                                </div>
                            </div>
                            :
                            <div className="p-grid">
                                <div className="col-lg-2 col-md-2 col-sm-2" style={{ margin: 'auto' }}>
                                    <i className="pi pi-check-circle pi-large pi-success"></i>
                                </div>
                                <div className="col-lg-10 col-md-10 col-sm-10">
                                    {this.state.dialog.detail}
                                </div>
                            </div>
                        }
                    </Dialog>
                </div>
                <CustomDialog type="confirmation" visible={this.state.showDialog} width="40vw"
                    header={'Edit System Event'} message={'Do you want to discard your changes? Your changes to the System Event will not be saved.'}
                    content={''} onClose={this.close} 
                    actions={ [ {id:"yes", title: 'Discard', callback: this.cancelEdit, className:'act-btn-dispose'},
                        {id:"no", title: 'Cancel', className:'act-btn-cancel', callback: this.close} ]}  >
                </CustomDialog>
            </React.Fragment>
        );
    }
}