import moment from "moment/moment";
import UIConstants from "../../../utils/ui.constants";
import React from "react";

function getFieldGroup(index, labelName, value, formatTime) {
    if (labelName === "Status") {
        value = <span className={"tag-content p-chips-token-label su-" + value} data-testid="status-badge">{value}</span>
    }
    if (formatTime) {
        value = moment.utc(value).format(UIConstants.CALENDAR_DATETIME_FORMAT)
    }

    return <div key={labelName + "-" + index} className="group" data-testid={"group-" + labelName}>
        <label className="col-5">{labelName}:</label>
        <div className="col-5">{value}</div>
    </div>
}

/**
 * Super component for all types (su, task, reservation) that can show a summary for a timeline item
 * @param props fields: [{labelName: <string>, value: <string>, formatTime: <boolean>}*]
 * @return {JSX.Element}
 */
export default function DetailsSummary(props) {
    const {fields} = props;

    return <div className="timeline-details-overview">
        {fields.map((field, index) => getFieldGroup(index, field.labelName, field.value, field.formatTime))}
    </div>
}