import DetailsSummary from "./DetailsSummary";
import {render} from "@testing-library/react";
import {removeReact18ConsoleErrors} from "../../../utils/test.helper";
import moment from "moment";

removeReact18ConsoleErrors()

describe('DetailsSummary', () => {
    const fields = [
        {labelName: 'X-men', value: 123, formatTime: false},
        {labelName: 'Status', value: 'Active', formatTime: false},
        {labelName: 'Date', value: '2023-07-28T12:34:56Z', formatTime: true},
    ];

    it('renders the correct number of field groups', () => {
        const {container} = render(<DetailsSummary fields={fields}/>);
        const fieldGroups = container.querySelectorAll('.group');
        expect(fieldGroups).toHaveLength(fields.length);
    });

     it('renders the normal field correctly', () => {
        const {getByText, getByTestId} = render(<DetailsSummary fields={fields}/>);
        expect(getByText(fields[0].labelName + ":")).toBeInTheDocument();
        expect(getByTestId('group-X-men')).toContainElement(getByText('123'));
    });

    it('renders the status field correctly', () => {
        const {getByText, getByTestId} = render(<DetailsSummary fields={fields}/>);
        expect(getByText(fields[1].labelName + ":")).toBeInTheDocument();
        expect(getByTestId('group-Status')).toContainElement(getByText('Active'));
        expect(getByTestId('status-badge')).toHaveClass("tag-content p-chips-token-label su-Active");
    });

    it('renders the date field correctly with formatted time', () => {
        const {getByText, getByTestId} = render(<DetailsSummary fields={fields}/>);
        const dateGroup = getByTestId('group-Date');
        expect(getByText(fields[2].labelName + ":")).toBeInTheDocument();
        const expectedFormattedTime = moment.utc(fields[2].value).format('YYYY-MM-DD HH:mm:ss');
        expect(dateGroup.textContent).toBe(`Date:${expectedFormattedTime}`
        );
    });
});