import React from "react";

export function getFieldsHtml(fields, status = undefined, style = {}) {
    const labelClassName = status ? `col-5 su-${status}-icon` : 'col-5'

    const result = 
     fields.map((field, index) => (
        <div key={field.label + "-" + index} className="entry">
            <label className={labelClassName} style={style}>{field.label}</label>
            <div className="col-7 initial-line-height">{field.value?field.value:'-'}</div>
        </div>
    ))
    return result;
}

const labelObject = (label, value) => ({
    label: label,
    value: value,
})

function getScheduleItem(mouseOverItem) {
    const scheduleItemFields = [
        labelObject("Project:", mouseOverItem.project),
        labelObject("Scheduling Unit:", mouseOverItem.name),
        labelObject("Scheduler:", mouseOverItem.scheduler),
        labelObject("Start Time:", mouseOverItem.real_start_time),
        labelObject("End Time:", mouseOverItem.real_end_time),
        labelObject("Antenna Set:", mouseOverItem.antenna_set),
        labelObject("Stations:", mouseOverItem.stations),
        labelObject("Status:", mouseOverItem.status),
        labelObject("Exposure Time:", mouseOverItem.on_sky_duration),
        labelObject("Duration:", mouseOverItem.duration),

    ];
    if (mouseOverItem.status === 'unschedulable') {
        scheduleItemFields.push({label: "Reason:", value: mouseOverItem.unschedulable_reason})
    }
    const suId = mouseOverItem.suId ? mouseOverItem.suId : mouseOverItem.id;
    const fieldsHTML = getFieldsHtml(scheduleItemFields, mouseOverItem.status)
    return (
        <div className={`p-grid su-${mouseOverItem.status} timeline-item-popover`}>
            <h3 className={`col-12 su-${mouseOverItem.status}-icon`}>
                Scheduling Unit ({suId}) Overview
            </h3>
            {fieldsHTML}
        </div>
    );
}

function getReservationItem(mouseOverItem) {
    let reservationItemFields = [
        labelObject("Name:", mouseOverItem.name),
        labelObject("Description:", mouseOverItem.desc),
        labelObject("Type:", mouseOverItem.activity_type),
        labelObject("Stations:", mouseOverItem.stations,),
        labelObject("Project:", mouseOverItem.project ? mouseOverItem.project : "-"),
        labelObject("Start Time:", mouseOverItem.real_start_time),
        labelObject("End Time:", mouseOverItem.real_end_time),
        labelObject("Duration:", mouseOverItem.duration),
        labelObject("Planned:", mouseOverItem.planned ? "Yes" : "No"),
    ];

    const reservationStyle = {
        width: '350px',
        backgroundColor: mouseOverItem.bgColor,
        color: mouseOverItem.color
    };
    return <div className={`p-grid timeline-item-popover`} style={reservationStyle}>
        <h3 className={`col-12`}>{"Reservation (" + mouseOverItem.reservationId + ") Overview"}</h3>
        {getFieldsHtml(reservationItemFields, undefined, {color: mouseOverItem.color})}
    </div>;
}

/**
 * @param mouseOverItem the item with all it's information to be shown in the pop-up
 * @return {*|null} the specific html for the type that is hovered over
 */
function getItemForType(mouseOverItem) {
    switch (mouseOverItem.type) {
        case "SCHEDULE":
            return getScheduleItem(mouseOverItem)
        case "RESERVATION":
            return getReservationItem(mouseOverItem)
        default: //when there should not be a pop-over; like a sun timing
            return null;
    }
}

/**
 * In the timeline, when hovering over an item, this popover will be shown
 * @param props mouseOverItem: the item with all it's information to be shown in the pop-up
 */
export default function TimelineItemPopover(props) {
    const {mouseOverItem} = props;
    if (!mouseOverItem) {
        return;
    }
    return <div className="p-overlaypanel-content">
        {getItemForType(mouseOverItem)}
    </div>
}
