import React, {useEffect, useState} from "react";
import UtilService from "../../../../services/util.service";
import moment from "moment";
import UIConstants from "../../../../utils/ui.constants";

import { ProgressSpinner } from 'primereact/progressspinner';
import Clock from 'react-live-clock';

function getDateOrTimeElement(title, element) {
    return <div className="element" data-testid={title + "-element"}>
        <label>{title}:</label>
        {element}
    </div>;
}

export default function DateTimeInfo() {
    const [currentUTC, setCurrentUTC] = useState()
    const [currentLST, setCurrentLST] = useState()


    useEffect(() => { // initializer
        if (!currentUTC) {
            async function fetchUTC() {
                let utc = await UtilService.getUTC()
                setCurrentUTC(moment(utc))
            }

            fetchUTC().catch(e => console.error("Couldn't get current utc from the server", e))
        }
    }, [])

    useEffect(() => { //get LST after current utc is retrieved
        async function fetchLST(currentUTC) {
            let lst = await UtilService.getLST(currentUTC.format(UIConstants.UTC_DATE_TIME_FORMAT))
            const hoursMinutesSeconds = lst.split(".")[0].split(":").map(stringTimeElement => parseInt(stringTimeElement))
            setCurrentLST(moment().set({
                hour: hoursMinutesSeconds[0], minute: hoursMinutesSeconds[1], second: hoursMinutesSeconds[2]
            }))
        }

        if (!currentLST && currentUTC) {
            fetchLST(currentUTC).catch(e => console.error("Couldn't get current lst from the server", e))
        }
    }, [currentUTC])

    if (!currentUTC || !currentLST) {
        return <div className="p-grid timeline-datetime-navigator">
            <ProgressSpinner className="m-4" style={{color: "var(--primary-300)"}}/>
        </div>
    }

    if (currentUTC.isValid() && currentLST.isValid()) {
        return <div className="datetime-info-container">
            {getDateOrTimeElement("Date", <span className="h5"
                                                title={currentUTC.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)}>
                    {currentUTC.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)}
                </span>)}
            {getDateOrTimeElement("UTC", <Clock className="h5"
                                                date={currentUTC.format(UIConstants.CALENDAR_DATETIME_FORMAT)}
                                                format="HH:mm:ss"
                                                interval={1000}
                                                ticking={true}/>)}
            {getDateOrTimeElement("LST", <Clock className="h5"
                                                date={currentLST.format(UIConstants.CALENDAR_DATETIME_FORMAT)}
                                                format="HH:mm:ss"
                                                interval={1000}
                                                ticking={true}/>)}
        </div>
    } else {
        return <div className="datetime-info-container" data-testid="errored-datetime-info">Something went wrong in retrieving the UTC and LST</div>
    }
}