import DateTimeInfo from "./DateTimeInfo";
import {render} from "@testing-library/react";
import UtilService from "../../../../services/util.service";
import {removeReact18ConsoleErrors} from "../../../../utils/test.helper";
import {act} from "react-dom/test-utils";

removeReact18ConsoleErrors()

jest.mock('../../../../services/util.service');

describe('DateTimeInfo Component', () => {
    beforeEach(() => {
        jest.clearAllMocks();
    });

    it('renders loading spinner when currentUTC and currentLST are not available', () => {
        const {getByRole} = render(<DateTimeInfo/>);
        expect(getByRole('alert',{ busy: true })).toBeInTheDocument();
    });

    it('renders datetime info when currentUTC and currentLST are available and valid', async () => {
        const utcTime = '2023-08-11 06:20:45';
        const lstTime = '2023-08-11 12:20:45';

        UtilService.getUTC.mockResolvedValue(utcTime);
        UtilService.getLST.mockResolvedValue(lstTime);

        let content
        await act(async () => {
            content = render(<DateTimeInfo/>);
            await new Promise((r) => setTimeout(r, 500));
        });

        expect(content.getByTestId('Date-element')).toBeInTheDocument();
        expect(content.getByTestId('UTC-element')).toBeInTheDocument();
        expect(content.getByTestId('LST-element')).toBeInTheDocument();

        expect(UtilService.getUTC).toHaveBeenCalled();
        expect(UtilService.getLST).toHaveBeenCalled();
    });

    it('renders error message when currentUTC or currentLST are not valid', async () => {
        const consoleWarnSpy = jest.spyOn(console, 'warn').mockImplementation(() => {});
        UtilService.getUTC.mockResolvedValue('invalid_utc');
        UtilService.getLST.mockResolvedValue('invalid_lst');

        let content
        await act(async () => {
            content = render(<DateTimeInfo/>);
            await new Promise((r) => setTimeout(r, 500));
        });

        expect(content.getByTestId('errored-datetime-info')).toBeInTheDocument();
        expect(consoleWarnSpy).toHaveBeenCalled();

        consoleWarnSpy.mockRestore();
    });
});