import Flatpickr from "react-flatpickr";
import {Button} from "primereact/button";
import React, {useEffect, useState} from "react";
import DateTimeInfo from "./DateTimeInfo";
import moment from "moment";
import UIConstants from "../../../../utils/ui.constants";
import {InputNumber} from "primereact/inputnumber";
import axios from "axios";
import {getStore} from "../../../../services/store.helper";

function DateSelector(props) {  //TODO: extract to separate class
    const {
        isRange,
        startTime,
        setStartAndEndTimeCallback
    } = props

    if (!startTime) {
        return null;
    }
    const title = isRange ? "Showing Date Range" : "Showing week from";
    const mode = isRange ? "range" : "single"
    const options = {
        "allowInput": true,
        "enableSeconds": isRange,
        "enableTime": isRange,
        "time_24hr": isRange,
        "mode": mode,
        "minuteIncrement": 1,
        "defaultHour": 0,
        "wrap": true,
        "inlineHideInput": true,
    }
    const resetButtonTitle = isRange ? "Reset to the default date range" : "Reset to the current week"

    return <div data-testid="nav-datetime-selector">
        <label>{title}</label>
        <Flatpickr data-enable-time={isRange}
                   data-input
                   value={startTime.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)}
                   options={options}
                   onClose={value => {
                       const chosenDate = moment(value[0]);
                       const newStartTime = moment(chosenDate.startOf('day'))
                       const newEndTime = moment(chosenDate.add(UIConstants.WEEKVIEW_SHOW_NUMBER_OF_DAYS, 'days').endOf('day'))
                       setStartAndEndTimeCallback(newStartTime, newEndTime)

                   }}>
            <input type="text" data-input className="p-inputtext p-component calendar-input"/>
            <Button className="p-button-icon-only calendar-button"
                    data-testid="nav-reset-button"
                    onClick={() => {
                        const now = moment(moment.now())
                        const newStartTime = moment(now.startOf('day'));
                        const newEndTime = moment(now.add(UIConstants.WEEKVIEW_SHOW_NUMBER_OF_DAYS, 'days').endOf('day'));
                        setStartAndEndTimeCallback(newStartTime, newEndTime)
                    }}
                    title={resetButtonTitle}>
                <i className="fas fa-undo"></i>
            </Button>
        </Flatpickr>
    </div>
}

function WeekChanger(props) {  //TODO: extract to separate class
    const {
        isRange,
        startTime,
        endTime,
        setStartAndEndTimeCallback
    } = props

    if (isRange || !startTime || !endTime) {
        return null
    }
    const weekNumbers = startTime.week() === endTime.week() ? startTime.week() : startTime.week() + "," + endTime.week()

    return <div className="week-changer" data-testid="nav-week-changer">
        <label>Week: <span className="text-normal">{weekNumbers}</span> </label>
        <div>
            <Button className="p-button-icon-only week-button"
                    data-testid="nav-week-previous"
                    icon="pi pi-angle-double-left"
                    title="Previous 7 days"
                    onClick={() => {
                        const newStartTime = moment(startTime).startOf('day').subtract(1, 'weeks');
                        const newEndTIme = moment(endTime).endOf('day').subtract(1, 'weeks');
                        setStartAndEndTimeCallback(newStartTime, newEndTIme)

                    }}/>

            <Button className="p-button-icon-only week-button"
                    data-testid="nav-week-next"
                    icon="pi pi-angle-double-right"
                    title="Next 7 days"
                    onClick={() => {
                        const newStartTime = moment(startTime).startOf('day').add(1, 'weeks');
                        const newEndTIme = moment(endTime).endOf('day').add(1, 'weeks');
                        setStartAndEndTimeCallback(newStartTime, newEndTIme)
                    }}/>
        </div>

    </div>
}

function JumpToSUId(props) { //TODO: extract to separate class
    const {
        suId,
        setSUId,
        setSearchForSUIdTrigger
    } = props

    return <div data-testid="nav-jump-to-suid">
        <label>Jump to SU id</label>
        <div>
            <InputNumber data-testid="jump-to-input"
                         className="jump-to-input"
                         value={suId}
                         format={false}
                         onValueChange={(e) => setSUId(e.value)}
            />
            <Button className="p-button-icon-only calendar-button"
                    title="Set the week to the scheduling unit's on sky or process start time."
                    data-testid="nav-jump-to-button"
                    onClick={() => setSearchForSUIdTrigger(prevState => !prevState)}>
                <i className="fas fa-search"></i>
            </Button>
        </div>
    </div>
}

function changeStartAndEndTimeCallback(setStartTime, setEndTime) {
    return (newStartTime, newEndTime) => {
        setStartTime(newStartTime);
        setEndTime(newEndTime);
    };
}

export default function DateTimeNavigator(props) {
    const {
        startTime,
        endTime,
        setStartTime,
        setEndTime
    } = props


    const [suIdInput, setSUIdInput] = useState()
    const [searchForSUIdTrigger, setSearchForSUIdTrigger] = useState(false)

    useEffect(() => {
        async function fetchStartTimeOfSUId(suId) {
            const shouldUseOnSkyTimes = getStore(UIConstants.STORE_KEY_TIMELINE).onSkyToggle ?? true
            const timeField = shouldUseOnSkyTimes ? "on_sky_start_time" : "process_start_time"
            const response = await axios.get('/api/scheduling_unit_blueprint/' + suId + "?fields=" + timeField);
            const newStartTime = moment(response.data[timeField]).startOf('day')
            const newEndTime = moment(moment(newStartTime).add(UIConstants.WEEKVIEW_SHOW_NUMBER_OF_DAYS, 'days').endOf('day'))
            if (newStartTime.isValid() && newEndTime.isValid()) {
                setStartTime(newStartTime);
                setEndTime(newEndTime);
            } else {
                console.error("Invalid start/end times found when jumping to the SU id's week.")
            }
        }

        if (suIdInput) {
            fetchStartTimeOfSUId(suIdInput).catch(e => console.error("Couldn't fetch the start time for SU id:", suIdInput, e))
        }
    }, [searchForSUIdTrigger])

    return <div className="p-grid timeline-datetime-navigator">
        <div className="section">
            <div className="header">Navigation</div>
            <div className="group">
                <div className="info-container">
                    <DateTimeInfo/>
                    <WeekChanger isRange={false} startTime={startTime} endTime={endTime}
                                 setStartAndEndTimeCallback={changeStartAndEndTimeCallback(setStartTime, setEndTime)}/>
                </div>
                <div className="selector-container">
                    <DateSelector isRange={false} startTime={startTime}
                                  setStartAndEndTimeCallback={changeStartAndEndTimeCallback(setStartTime, setEndTime)}/>
                    <JumpToSUId suId={suIdInput} setSUId={setSUIdInput}
                                setSearchForSUIdTrigger={setSearchForSUIdTrigger}/>
                </div>
            </div>
        </div>
    </div>
}