import moment from "moment";
import DateTimeNavigator from "./DateTimeNavigator";
import {fireEvent, render} from "@testing-library/react";
import {clickItem, removeReact18ConsoleErrors} from "../../../../utils/test.helper";

removeReact18ConsoleErrors()

describe('DateTimeNavigator', () => {
    const mockSetStartTime = jest.fn();
    const mockSetEndTime = jest.fn();

    it('renders date selector, jump to and week changer correctly', () => {
        const pageContent = render(
            <DateTimeNavigator
                startTime={moment()}
                endTime={moment()}
                setStartTime={mockSetStartTime}
                setEndTime={mockSetEndTime}
            />
        );

        const dateSelector = pageContent.getByTestId('nav-datetime-selector');
        const resetButton = pageContent.getByTestId('nav-reset-button');
        const nextWeekButton = pageContent.getByTestId('nav-week-next');
        const previousWeekButton = pageContent.getByTestId('nav-week-previous');
        const jumpToSUIdButton = pageContent.getByTestId('nav-jump-to-button');

        expect(dateSelector).toBeInTheDocument();
        expect(dateSelector.textContent).toBe("Showing week from");
        expect(resetButton).toBeInTheDocument();
        expect(resetButton.title).toBe("Reset to the current week");
        expect(nextWeekButton).toBeInTheDocument();
        expect(nextWeekButton.title).toBe("Next 7 days");
        expect(previousWeekButton).toBeInTheDocument();
        expect(previousWeekButton.title).toBe("Previous 7 days");
        expect(jumpToSUIdButton).toBeInTheDocument();
        expect(jumpToSUIdButton.title).toBe("Set the week to the scheduling unit's on sky or process start time.")
    });

    it('Clicks buttons; reset, previous+next week', () => {
        const mockStartTime = moment('2023-08-15');
        const mockEndTime = moment('2023-08-22');
        const pageContent = render(
            <DateTimeNavigator
                startTime={mockStartTime}
                endTime={mockEndTime}
                setStartTime={mockSetStartTime}
                setEndTime={mockSetEndTime}
            />
        );

        const resetButton = pageContent.getByTestId('nav-reset-button');
        const nextWeekButton = pageContent.getByTestId('nav-week-next');
        const previousWeekButton = pageContent.getByTestId('nav-week-previous');


        clickItem(resetButton)
        expect(mockSetStartTime).toHaveBeenCalledTimes(1);
        expect(mockSetEndTime).toHaveBeenCalledTimes(1);

        clickItem(previousWeekButton)
        expect(mockSetStartTime).toHaveBeenCalledTimes(2);
        expect(mockSetEndTime).toHaveBeenCalledTimes(2);

        clickItem(nextWeekButton)
        expect(mockSetStartTime).toHaveBeenCalledTimes(3);
        expect(mockSetEndTime).toHaveBeenCalledTimes(3);
    });

    
    // TODO : Fix Unit Test Sets the start time for an SU ID upon search click
    it.skip('Sets the start time for an SU ID upon search click', () => {
        const pageContent = render(
            <DateTimeNavigator
                startTime={moment('2023-08-15')}
                endTime={moment('2023-08-22')}
                setStartTime={mockSetStartTime}
                setEndTime={mockSetEndTime}
            />
        );

        const suIdInput = pageContent.getByTestId('jump-to-input');
        const jumpToButton = pageContent.getByTestId('nav-jump-to-button');

        const suId = 456
        clickItem(suIdInput)
        fireEvent.change(suIdInput.querySelector('input'), {target: {value: suId}});
        clickItem(jumpToButton)

        expect(mockSetStartTime).toHaveBeenCalledWith(expect.any(Date));
    })

    it.skip('does not render week changer or date selector when startTime is not provided', () => {
        const pageContent = render(
            <DateTimeNavigator
                startTime={undefined}
                endTime={moment()}
                setStartTime={mockSetStartTime}
                setEndTime={mockSetEndTime}
            />
        );

        expect(pageContent.queryByTestId('nav-datetime-selector')).not.toBeInTheDocument()
        expect(pageContent.queryByTestId('nav-week-changer')).not.toBeInTheDocument()
    });
});