import * as HelperFunctions from "../../helpers/toolbar/zoomandmove.helper";
import ZoomAndMove from "./ZoomAndMove";
import {render} from "@testing-library/react";
import {clickItem, removeReact18ConsoleErrors} from "../../../../utils/test.helper";
import UIConstants from "../../../../utils/ui.constants";

removeReact18ConsoleErrors()

jest.mock('../../../../services/store.helper', () => ({
    updateStore: jest.fn(),
}));
jest.mock('../../helpers/toolbar/zoomandmove.helper', () => ({
    moveTimeline: jest.fn(),
    getZoomPossibilities: jest.fn(),
    getMovePossibilities: jest.fn(),
    getZoomTimes: jest.fn(),
    getTimeStepsForZoom: jest.fn(),
}));

describe('ZoomAndMove', () => {
    const mockTimelineStore = {
        zoomLevel: undefined,
    };
    const mockSetVisibleTime = jest.fn();
    const mockSetVisibleEndTime = jest.fn();
    const mockVisibleStartTime = 'mockVisibleStartTime';
    const mockVisibleEndTime = 'mockVisibleEndTime';
    const mockSetTimeSteps = jest.fn()
    const mockSetHeaderUnit = jest.fn()

    beforeEach(() => {
        HelperFunctions.getZoomTimes.mockImplementation(() => {
            return {start: mockVisibleStartTime, end: mockVisibleEndTime}
        });
        HelperFunctions.getZoomPossibilities.mockImplementation(() => {
            return {canZoomIn: true, canZoomOut: true}
        });
        HelperFunctions.getMovePossibilities.mockImplementation(() => {
            return {moveLeft: true, moveRight: true}
        })
        HelperFunctions.getTimeStepsForZoom.mockReturnValue(() => UIConstants.ALL_TIMESTEPS[0]);
    });

    afterEach(() => {
        jest.clearAllMocks();

    })
    

    it('renders loading spinner when props are not yet available', () => {
        let moctm = mockTimelineStore;
       const {getByRole}= render(
            <ZoomAndMove
                timelineStore={moctm}
            />
        );

        expect(getByRole('alert',{ busy: true })).toBeInTheDocument();
    });
    

  
 // TODO : Fix unit test renders components and clicks them when data is available
    it.skip('renders components and clicks them when data is available', () => {
        const defaultTimeSteps = UIConstants.ALL_TIMESTEPS[0]
        // moveTimeline.mockImplementation((_, minuteAmount) => {
        //     if (minuteAmount === -30) {
        //         mockSetVisibleTime.mockReturnValue(mockVisibleStartTime);
        //         mockSetVisibleEndTime.mockReturnValue(mockVisibleEndTime);
        //     }
        // });

        jest.spyOn(HelperFunctions, 'getTimeStepsForZoom').mockImplementation(() => {
            // Mock implementation of getTimeStepsForZoom here
            return UIConstants.ALL_TIMESTEPS[0]
        });


        const pageContent = render(
            <ZoomAndMove
                timelineStore={mockTimelineStore}
                visibleTime={mockSetVisibleTime}
                setVisibleTime={mockSetVisibleTime}
                visibleEndTime={mockSetVisibleEndTime}
                setVisibleEndTime={mockSetVisibleEndTime}
                timeSteps={defaultTimeSteps}
                setTimeSteps={mockSetTimeSteps}
                setHeaderUnit={mockSetHeaderUnit}
            />
        );

        expect(pageContent.getByTestId('zoom-select')).toBeInTheDocument();
        clickItem(pageContent.getByTestId('Zoom out'))
        expect(mockSetVisibleTime).toHaveBeenNthCalledWith(1, mockVisibleStartTime);
        expect(mockSetVisibleEndTime).toHaveBeenNthCalledWith(1, mockVisibleEndTime);
        clickItem(pageContent.getByTestId('Move Left'))
        expect(mockSetVisibleTime).toHaveBeenNthCalledWith(2, mockVisibleStartTime);
        expect(mockSetVisibleEndTime).toHaveBeenNthCalledWith(2, mockVisibleEndTime);
        clickItem(pageContent.getByTestId('zoom-reset-button'))
        expect(mockSetVisibleTime).toHaveBeenNthCalledWith(3, mockVisibleStartTime);
        expect(mockSetVisibleEndTime).toHaveBeenNthCalledWith(3, mockVisibleEndTime);
        //zoom-time-selector
        //time-steps-setter
    });
    
});