import ScheduleService from "../../../services/schedule.service";
import ProjectService from "../../../services/project.service";
import UtilService from "../../../services/util.service";


async function fetchReservationTemplatesGetReasons() {
    let reasons = []
    UtilService.getReservationTemplates().then(templates => {
        const reservationTemplate = templates[0] ?? null; //use the first if there are templates found
        if (reservationTemplate) {
            let foundTemplateReason = reservationTemplate.ref_resolved_schema?.properties?.activity?.properties?.type?.enum;
            foundTemplateReason.forEach(reason => {
                reasons.push({name: reason})
            })
        }
    });
    return reasons
}

async function fetchSchedulingUnitStatuses() {
    const suFilters = await ScheduleService.getSchedulingUnitFilterDefinition('blueprint');
    return suFilters?.data?.filters['status']?.choices.map(suStatusChoice => {
        return {name: suStatusChoice.value, value: suStatusChoice.value}
    })
}

export async function fetchAllOptionsForMultiSelects(setMultiSelectAllOptions) {
    setMultiSelectAllOptions(prevState => ({
        ...prevState,
        isLoading: true,
    }))

    const suStatuses = await fetchSchedulingUnitStatuses()
    const reasons = await fetchReservationTemplatesGetReasons();
    const projectNamesWithState = await ProjectService.getProjectList(false, "name,project_state_value")
    const projectStatuses = await ProjectService.getProjectStates()

    setMultiSelectAllOptions({
        isLoading: false,
        reservationReasons: reasons,
        schedulingUnitStatuses: suStatuses,
        projectNamesWithState: projectNamesWithState,
        projectStatuses: projectStatuses
    })
}