import React from "react";

/**
 * Shifts the header intervals for the LST time header
 * Library's documentation about date header renderers: https://github.com/namespace-ee/react-calendar-timeline#dateheader
 * @param getIntervalProps mandatory by library
 * @param intervalContext mandatory by library
 * @param lstShiftInSeconds based on the visible start time of the Timeline in utc, the corresponding lst time is retrieved
 * and the approximate shift between the two is calculated for the header intervals. It is not a 100% correct, but close enough
 * @return {JSX.Element}
 */
export function getIntervalRendererLST({getIntervalProps, intervalContext}, lstShiftInSeconds) {
    let lstTime = "-"
    let startTime = null;
    
    if (lstShiftInSeconds !== undefined && intervalContext?.interval?.startTime!==undefined) {
        startTime = intervalContext?.interval?.startTime.clone();
        lstTime = startTime.add(lstShiftInSeconds, 'seconds')
        lstTime = lstTime.format("HH:mm")
    }

    return <div {...getIntervalProps()} className="rct-dateHeader" style={getIntervalProps().style}
                data-testid={"dateheader-lst-" + lstTime}>
        {lstTime}
    </div>;
}

/**
 * The Timeline library takes this timeSteps object as a property to display the columns nicely (including the header).
 * This function makes sure that it changes accordingly to the visible start- and endtime in the Timeline
 * @param durationInHours based on this duration, the header time steps should change
 * @return {{hour: number, month: number, year: number, day: number, second: number, minute: number}}
 */
export function getTimeSteps(durationInHours) {
    let timeSteps = {
        second: 15,
        minute: 5,
        hour: 0.5,
        day: 1,
        month: 1,
        year: 1
    };

    if (durationInHours <= 0.5) {
        timeSteps.minute = 1
    } else if (durationInHours < 2) {
        timeSteps.minute = 5
    } else if (durationInHours < 4) {
        timeSteps.minute = 10
    } else if (durationInHours > 12) {
        timeSteps.hour = 1
    }

    return timeSteps
}