import React from "react";

function getItemDivStyle(itemContext, item, hasItemColors = false) {
    const style = {
        borderRadius: 3,
        border: 'none',
        overflow: "hidden",
        textOverflow: "ellipsis",
    };
    if (hasItemColors) {
        style.color = item.color
        style.background = itemContext.selected ? 'none' : item.bgColor
        style.opacity = item.opacity ? item.opacity : 100
    }
    return style;
}

function getItemContentStyle(itemContext,
                             lineHeightScale = 2,
                             fontSize = 14) {
    return {
        lineHeight: `${Math.floor(itemContext.dimensions.height / lineHeightScale)}px`,
        fontSize: `${fontSize}px`,
        textAlign: "center",
    };
}

function calculatePopPosition(evt) {
    let popPosition = {
        display: "block",
        left: `${evt.pageX + 400 > window.innerWidth ? evt.pageX - 400 : evt.pageX + 20}px`
    };
   
    if (evt.clientY > window.screen.height / 2) {
     //   popPosition.bottom = `${evt.clientY - evt.pageY + 30}px`;  
        popPosition.top = `${evt.pageY - 400}px`; // Bottem line position will not work here
    } else {
        popPosition.top = `${evt.pageY}px`;
    }
    return popPosition;
}

function hoverOverItem(evt, item, setPopPositionCallback, setMouseOverItemCallback) {
    let popPosition = calculatePopPosition(evt);
    setMouseOverItemCallback(item)
    setPopPositionCallback(popPosition)
}

function hoverOutItem(setPopPositionCallback) {
    setPopPositionCallback({display: 'none'})
}

function renderSunTimingItem({
                                 item,
                                 itemContext,
                                 getItemProps
                             }, setMouseOverItemCallback) {
    itemContext.dimensions.height = 20

    return <div {...getItemProps({style: {...getItemDivStyle(itemContext, item, true), height: "90%"}})}
                onMouseOver={(evt) => {
                    hoverOverItem(evt, item, () => {
                    }, setMouseOverItemCallback)
                }}
    >
    </div>
}

function renderReservationItem({
                                   item,
                                   itemContext,
                                   getItemProps
                               }, setPopPositionCallback, setMouseOverItemCallback, setSummaryItemCallback) {
    let itemContentStyle = getItemContentStyle(itemContext);

    return <div {...getItemProps({style: getItemDivStyle(itemContext, item, true)})}
                onMouseDown={() => setSummaryItemCallback({id: item.reservationId, type: item.type})}
                onMouseOver={(evt) => {
                    hoverOverItem(evt, item, setPopPositionCallback, setMouseOverItemCallback)
                }}
                onMouseOut={() => {
                    hoverOutItem(setPopPositionCallback)
                }}
    >
        <div style={itemContentStyle}>
            {item.activity_type}
            <div>{item.desc}</div>
        </div>
    </div>
}

function renderSUItem({
                          item,
                          itemContext,
                          getItemProps
                      }, setPopPositionCallback, setMouseOverItemCallback, setSummaryItemCallback) {
    const scheduleMethodBorder = item.scheduler === "dynamic" ? "1.5px dashed black" : "1.5px solid black"
    let itemDivStyle = getItemDivStyle(itemContext, item);
    itemDivStyle.fontWeight = 600
    itemDivStyle.border = scheduleMethodBorder

    let itemContentStyle = getItemContentStyle(itemContext, 3, 12)
    let itemContent = `${item.project} - ${item.suId ? item.suId : item.id} - ${item.name} - ${item.band} - ${item.on_sky_duration}`

    return <div {...getItemProps({
        style: itemDivStyle,
        className: `${item.status ? 'su-' + item.status : ''}`,
    })}
                onMouseDown={() => setSummaryItemCallback({id: item.suId, type: item.type})}
                onMouseOver={(evt) => {
                    hoverOverItem(evt, item, setPopPositionCallback, setMouseOverItemCallback)
                }}
                onMouseOut={() => {
                    hoverOutItem(setPopPositionCallback)
                }}
    >
        <div style={itemContentStyle}>
            {itemContent}
        </div>
    </div>
}

/** Custom function to pass to timeline component to render item */
export function itemRenderer({item, timelineContext, itemContext, getItemProps, getResizeProps},
                             setPopPositionCallback, setMouseOverItemCallback, setSummaryItemCallback) {
    switch (item.type) {
        case "RESERVATION":
            return renderReservationItem({
                item,
                itemContext,
                getItemProps
            }, setPopPositionCallback, setMouseOverItemCallback, setSummaryItemCallback)
        case "SCHEDULE":
            return renderSUItem({
                item,
                itemContext,
                getItemProps
            }, setPopPositionCallback, setMouseOverItemCallback, setSummaryItemCallback)
        default: //a sun timings
            return renderSunTimingItem({
                item,
                itemContext,
                getItemProps
            }, setMouseOverItemCallback)
    }
}

export function groupRenderer({group}) {
    const titleSplitOnWeek = group.title.split(":");
    const week = titleSplitOnWeek[0]
    const date = titleSplitOnWeek[1]
    return <div className="group-renderer">
        <span className="week">{week}</span>
        {date}
        {group.cursorInfo ?
            <div className="cursor-information">
                <span>UTC: {group.cursorInfo.utc}</span>
                <span>LST: {group.cursorInfo.lst}</span>
            </div> : null}
    </div>;
}