import React, { Component } from 'react';
import { Dropdown } from 'primereact/dropdown';
import WorkflowService from '../../services/workflow.service';
import { Link } from 'react-router-dom';
import { InputText } from 'primereact/inputtext';
import { appGrowl } from '../../layout/components/AppGrowl';
import Validator from '../../utils/validator';

/** Component to assign workflow tasks in the Workflow process */
class TaskAssigner extends Component {

    constructor(props) {
        super(props);
        this.state = { 
            errors: {}
        };
        this.user =JSON.parse(localStorage.getItem("user"));
        this.assignUser = this.assignUser.bind(this);
    }

    /**
     * Validate and set value for user amail field
     * @param {string} email 
     */
    setUserMail(email) {
        let errors = this.state.errors;
        if (email && !Validator.isValidEmail(email)) {
            errors['email'] = "Enter vaild email";
        }   else {
            delete errors['email'];
        }
        this.setState({assignToEmail: email, assignToRole: null, errors: errors});
    }

    /**
     * To assign or reassign user to the current workflow task. To assign to the logged in user 'Assign To Me' link is clcked 
     * and self assigned.
     * @param {boolean} selfAssign 
     */
    async assignUser(selfAssign) {
        let toUser = selfAssign?`self=true`:
                            (this.state.assignToRole?`project_role=${this.state.assignToRole}`
                                :(this.state.assignToEmail?`user_email=${this.state.assignToEmail}`:""));
        let assignStatus = null;
        if (this.props.currentWorkflowTask.owner_id) {
            if (toUser) {
                assignStatus = await WorkflowService.reAssignTo(this.props.currentWorkflowTask.id, toUser, {});
            }
        }   else {
            assignStatus =await WorkflowService.updateAssignTo(this.props.currentWorkflowTask.id, toUser, {});
        }
        if (assignStatus) {
            if (assignStatus.status !== "error") {
                // Callback to the parent component after assigning user to the task.
                this.props.callback();
            }   else {
                appGrowl.show({ severity: 'error', summary: 'Failed to assign', detail: JSON.stringify(assignStatus.error) });
            }
        }   
    }

    render() {
        const allowSave = this.state.assignToRole  || (this.state.assignToEmail && !this.state.errors.email);
        return (
            <>
                <div>
                    { 
                    this.props.currentWorkflowTask.editPermissions && !this.props.disableAssignToMe
                    ? <Link to="#" className="notDisabled" onClick={e => {this.assignUser(true)}}>Assign to me</Link>
                    : <Link to="#" style={{ cursor: 'disabled', pointerEvents: "none", color: 'grey'}}>Assign to me</Link>
                    }
                </div>
                <Dropdown disabled={!this.props.currentWorkflowTask.editPermissions} inputId="assignToValue" 
                        tooltip="Assign to user with role (will be assigned to the first user in this role)"
                        value={this.state.assignToRole} optionLabel="displayValue" optionValue="value" 
                        onChange={(e) => this.setState({assignToRole: e.value, assignToEmail: null})}
                        options={this.props.projectRoles}
                        placeholder="Project Role" style={{marginBottom: '5px'}} />
                <div style={{marginBottom: '5px'}} >
                    <InputText disable={!this.props.currentWorkflowTask.editPermissions}value={this.state.assignToEmail} placeholder="User Email" 
                        tooltip="Assign to user with this email"
                        onChange={e => this.setUserMail(e.target.value)} />
                    <label className="error">
                        {this.state.errors.email ? this.state.errors.email : ""}
                    </label>
                </div>
                <div style={{float: 'right'}}>
                    <Link to="#" onClick={e => {this.assignUser()}} style={{marginRight: '10px'}}
                        className={allowSave?'':'disabled-link'}>
                        <i className="fa fa-check"></i></Link>
                    <Link to="#" onClick={e => {this.props.callback()}}>
                    <i className="fa fa-times"></i></Link>
                </div>
            </>
        );
    }
}

export default TaskAssigner;